@@ -2,6 +2,7 @@
 
 from collections import defaultdict
 import os
+import string
 import json
 import time
 
@@ -15,10 +16,10 @@ def read_log():
         if start_token_seen:
             try:
                 dur, kind, test_id = line.split()
-            except:
+            except ValueError:
                 return
             else:
-                if dur[0] not in '0123456789':
+                if dur[0] not in string.digits:
                     return
             if kind != 'call':
                 continue
