@@ -34,12 +34,15 @@ git_whitelist = {
     '.mailmap',
     # CI
     '.github/PULL_REQUEST_TEMPLATE.md',
+    '.github/dependabot.yml',
     '.github/workflows/runtests.yml',
+    '.github/workflows/emscripten.yml',
     '.github/workflows/ci-sage.yml',
     '.github/workflows/comment-on-pr.yml',
     '.github/workflows/release.yml',
     '.github/workflows/docs-preview.yml',
     '.github/workflows/checkconflict.yml',
+    '.github/workflows/nightly-wheels.yml',
     '.ci/durations.json',
     '.ci/generate_durations_log.sh',
     '.ci/parse_durations_log.py',
@@ -97,7 +100,7 @@ git_whitelist = {
     'bin/mailmap_check.py',
     'bin/test_symengine.py',
     'bin/test_tensorflow.py',
-    'bin/test_pyodide.mjs',
+    'bin/test_pytorch.py',
     # The notebooks are not ready for shipping yet. They need to be cleaned
     # up, and preferably doctested.  See also
     # https://github.com/sympy/sympy/issues/6039.
