@@ -20,8 +20,7 @@ def main(version, outdir):
     out = '\n'.join(["%s\t%s" % (i, os.path.split(j)[1]) for i, j in out])
 
     # Output to file and to screen
-    with open(outdir / 'sha256.txt', 'w') as shafile:
-        shafile.write(out)
+    Path(outdir / 'sha256.txt').write_text(out)
     print(out)
 
 
