@@ -8,7 +8,9 @@ from sympy.core.kind import BooleanKind
 from sympy.core.relational import Eq, Ne, Gt, Lt, Ge, Le
 from sympy.logic.inference import satisfiable
 from sympy.utilities.decorator import memoize_property
-from sympy.utilities.exceptions import SymPyDeprecationWarning
+from sympy.utilities.exceptions import (sympy_deprecation_warning,
+                                        SymPyDeprecationWarning,
+                                        ignore_warnings)
 
 
 # Memoization is necessary for the properties of AssumptionKeys to
@@ -72,6 +74,11 @@ class AssumptionKeys:
         from .handlers.sets import IntegerPredicate
         return IntegerPredicate()
 
+    @memoize_property
+    def noninteger(self):
+        from .predicates.sets import NonIntegerPredicate
+        return NonIntegerPredicate()
+
     @memoize_property
     def rational(self):
         from .handlers.sets import RationalPredicate
@@ -347,6 +354,10 @@ def _extract_all_facts(assump, exprs):
                 else:
                     # If any of the literals doesn't have matching expr don't add the whole clause.
                     break
+            else:
+                # If any of the literals aren't unary predicate don't add the whole clause.
+                break
+
         else:
             if args:
                 facts.add(frozenset(args))
@@ -363,7 +374,7 @@ def ask(proposition, assumptions=True, context=global_assumptions):
     This function evaluates the proposition to ``True`` or ``False`` if
     the truth value can be determined. If not, it returns ``None``.
 
-    It should be discerned from :func:`~.refine()` which, when applied to a
+    It should be discerned from :func:`~.refine` which, when applied to a
     proposition, simplifies the argument to symbolic ``Boolean`` instead of
     Python built-in ``True``, ``False`` or ``None``.
 
@@ -379,14 +390,14 @@ def ask(proposition, assumptions=True, context=global_assumptions):
     Parameters
     ==========
 
-    proposition : Any boolean expression.
+    proposition : Boolean
         Proposition which will be evaluated to boolean value. If this is
         not ``AppliedPredicate``, it will be wrapped by ``Q.is_true``.
 
-    assumptions : Any boolean expression, optional.
+    assumptions : Boolean, optional
         Local assumptions to evaluate the *proposition*.
 
-    context : AssumptionsContext, optional.
+    context : AssumptionsContext, optional
         Default assumptions to evaluate the *proposition*. By default,
         this is ``sympy.assumptions.global_assumptions`` variable.
 
@@ -444,6 +455,8 @@ def ask(proposition, assumptions=True, context=global_assumptions):
         be determined.
     """
     from sympy.assumptions.satask import satask
+    from sympy.assumptions.lra_satask import lra_satask
+    from sympy.logic.algorithms.lra_theory import UnhandledInput
 
     proposition = sympify(proposition)
     assumptions = sympify(assumptions)
@@ -458,7 +471,7 @@ def ask(proposition, assumptions=True, context=global_assumptions):
     if isinstance(proposition, AppliedPredicate):
         key, args = proposition.function, proposition.arguments
     elif proposition.func in binrelpreds:
-        key, args = binrelpreds[proposition.func], proposition.args
+        key, args = binrelpreds[type(proposition)], proposition.args
     else:
         key, args = Q.is_true, (proposition,)
 
@@ -491,6 +504,14 @@ def ask(proposition, assumptions=True, context=global_assumptions):
 
     # using satask (still costly)
     res = satask(proposition, assumptions=assumptions, context=context)
+    if res is not None:
+        return res
+
+    try:
+        res = lra_satask(proposition, assumptions=assumptions, context=context)
+    except UnhandledInput:
+        return None
+
     return res
 
 
@@ -586,12 +607,14 @@ def register_handler(key, handler):
         Use multipledispatch handler instead. See :obj:`~.Predicate`.
 
     """
-    SymPyDeprecationWarning(
-        feature="register_handler() function",
-        useinstead="multipledispatch handler of Predicate",
-        issue=20873,
-        deprecated_since_version="1.8"
-    ).warn()
+    sympy_deprecation_warning(
+        """
+        The AskHandler system is deprecated. The register_handler() function
+        should be replaced with the multipledispatch handler of Predicate.
+        """,
+        deprecated_since_version="1.8",
+        active_deprecations_target='deprecated-askhandler',
+    )
     if isinstance(key, Predicate):
         key = key.name.name
     Qkey = getattr(Q, key, None)
@@ -603,21 +626,25 @@ def register_handler(key, handler):
 
 def remove_handler(key, handler):
     """
-    Removes a handler from the ask system. Same syntax as register_handler
+    Removes a handler from the ask system.
 
     .. deprecated:: 1.8.
         Use multipledispatch handler instead. See :obj:`~.Predicate`.
 
     """
-    SymPyDeprecationWarning(
-        feature="remove_handler() function",
-        useinstead="multipledispatch handler of Predicate",
-        issue=20873,
-        deprecated_since_version="1.8"
-    ).warn()
+    sympy_deprecation_warning(
+        """
+        The AskHandler system is deprecated. The remove_handler() function
+        should be replaced with the multipledispatch handler of Predicate.
+        """,
+        deprecated_since_version="1.8",
+        active_deprecations_target='deprecated-askhandler',
+    )
     if isinstance(key, Predicate):
         key = key.name.name
-    getattr(Q, key).remove_handler(handler)
+    # Don't show the same warning again recursively
+    with ignore_warnings(SymPyDeprecationWarning):
+        getattr(Q, key).remove_handler(handler)
 
 
 from sympy.assumptions.ask_generated import (get_all_known_facts,
