@@ -4,9 +4,10 @@ infinitesimal, finite, etc.
 """
 
 from sympy.assumptions import Q, ask
-from sympy.core import Add, Mul, Pow, Symbol
-from sympy.core.numbers import (ComplexInfinity, Exp1, GoldenRatio, ImaginaryUnit,
-    Infinity, NaN, NegativeInfinity, Number, Pi, TribonacciConstant, E)
+from sympy.core import Expr, Add, Mul, Pow, Symbol
+from sympy.core.numbers import (NegativeInfinity, GoldenRatio,
+    Infinity, Exp1, ComplexInfinity, ImaginaryUnit, NaN, Number, Pi, E,
+    TribonacciConstant)
 from sympy.functions import cos, exp, log, sign, sin
 from sympy.logic.boolalg import conjuncts
 
@@ -101,7 +102,7 @@ def _(expr, assumptions):
         # is None and Bounded is None or there was already
         # an unknown sign, return None
         if sign != -1 and s != sign or \
-                s is None and (s == _bounded or s == sign):
+                s is None and None in (_bounded, sign):
             return None
         else:
             sign = s
@@ -150,10 +151,14 @@ def _(expr, assumptions):
         * /s = not signed
     """
     result = True
+    possible_zero = False
     for arg in expr.args:
         _bounded = ask(Q.finite(arg), assumptions)
         if _bounded:
-            continue
+            if ask(Q.zero(arg), assumptions) is not False:
+                if result is False:
+                    return None
+                possible_zero = True
         elif _bounded is None:
             if result is None:
                 return None
@@ -162,6 +167,8 @@ def _(expr, assumptions):
             if result is not False:
                 result = None
         else:
+            if possible_zero:
+                return None
             result = False
     return result
 
@@ -188,6 +195,12 @@ def _(expr, assumptions):
     if base_bounded is False and ask(Q.extended_nonzero(expr.exp), assumptions):
         return False
     if base_bounded and exp_bounded:
+        is_base_zero = ask(Q.zero(expr.base),assumptions)
+        is_exp_negative = ask(Q.negative(expr.exp),assumptions)
+        if is_base_zero is True and is_exp_negative is True:
+            return False
+        if is_base_zero is not False and is_exp_negative is not False:
+            return None
         return True
     if (abs(expr.base) <= 1) == True and ask(Q.extended_positive(expr.exp), assumptions):
         return True
@@ -226,9 +239,12 @@ def _(expr, assumptions):
 # InfinitePredicate
 
 
-@InfinitePredicate.register_many(ComplexInfinity, Infinity, NegativeInfinity)
+@InfinitePredicate.register(Expr)
 def _(expr, assumptions):
-    return True
+    is_finite = Q.finite(expr)._eval_ask(assumptions)
+    if is_finite is None:
+        return None
+    return not is_finite
 
 
 # PositiveInfinitePredicate
