@@ -1,33 +1,44 @@
+from __future__ import annotations
+
 from sympy.calculus.singularities import is_decreasing
-from sympy.calculus.util import AccumulationBounds
-from sympy.concrete.expr_with_limits import AddWithLimits
-from sympy.concrete.expr_with_intlimits import ExprWithIntLimits
-from sympy.concrete.gosper import gosper_sum
+from sympy.calculus.accumulationbounds import AccumulationBounds
+from .expr_with_intlimits import ExprWithIntLimits
+from .expr_with_limits import AddWithLimits
+from .gosper import gosper_sum
+from sympy.core.expr import Expr
 from sympy.core.add import Add
-from sympy.core.function import Derivative
+from sympy.core.containers import Tuple
+from sympy.core.function import Derivative, expand
 from sympy.core.mul import Mul
+from sympy.core.numbers import Float, _illegal
 from sympy.core.relational import Eq
 from sympy.core.singleton import S
-from sympy.core.symbol import Dummy, Wild, Symbol
-from sympy.functions.special.zeta_functions import zeta
+from sympy.core.sorting import ordered
+from sympy.core.symbol import Dummy, Wild, Symbol, symbols
+from sympy.functions.combinatorial.factorials import factorial
+from sympy.functions.combinatorial.numbers import bernoulli, harmonic
+from sympy.functions.elementary.complexes import re
+from sympy.functions.elementary.exponential import exp, log
 from sympy.functions.elementary.piecewise import Piecewise
 from sympy.functions.elementary.trigonometric import cot, csc
-from sympy.logic.boolalg import And
-from sympy.polys import apart, together
+from sympy.functions.special.hyper import hyper
+from sympy.functions.special.tensor_functions import KroneckerDelta
+from sympy.functions.special.zeta_functions import zeta
+from sympy.integrals.integrals import Integral
+from sympy.logic.boolalg import And, Not
+from sympy.polys.partfrac import apart
 from sympy.polys.polyerrors import PolynomialError, PolificationFailed
-from sympy.polys.polytools import parallel_poly_from_expr
+from sympy.polys.polytools import parallel_poly_from_expr, Poly, factor
+from sympy.polys.rationaltools import together
 from sympy.series.limitseq import limit_seq
 from sympy.series.order import O
 from sympy.series.residues import residue
-from sympy.sets.sets import FiniteSet
-from sympy.simplify import denom
-from sympy.simplify.combsimp import combsimp
-from sympy.simplify.powsimp import powsimp
-from sympy.solvers import solve
-from sympy.solvers.solveset import solveset
+from sympy.sets.contains import Contains
+from sympy.sets.sets import FiniteSet, Interval
 from sympy.utilities.iterables import sift
 import itertools
 
+
 class Sum(AddWithLimits, ExprWithIntLimits):
     r"""
     Represents unevaluated summation.
@@ -156,12 +167,14 @@ class Sum(AddWithLimits, ExprWithIntLimits):
 
     .. [1] Michael Karr, "Summation in Finite Terms", Journal of the ACM,
            Volume 28 Issue 2, April 1981, Pages 305-350
-           http://dl.acm.org/citation.cfm?doid=322248.322255
+           https://dl.acm.org/doi/10.1145/322248.322255
     .. [2] https://en.wikipedia.org/wiki/Summation#Capital-sigma_notation
     .. [3] https://en.wikipedia.org/wiki/Empty_sum
     """
 
-    __slots__ = ('is_commutative',)
+    __slots__ = ()
+
+    limits: tuple[tuple[Symbol, Expr, Expr]]
 
     def __new__(cls, function, *symbols, **assumptions):
         obj = AddWithLimits.__new__(cls, function, *symbols, **assumptions)
@@ -213,7 +226,7 @@ class Sum(AddWithLimits, ExprWithIntLimits):
         if reps:
             undo = {v: k for k, v in reps.items()}
             did = self.xreplace(reps).doit(**hints)
-            if type(did) is tuple:  # when separate=True
+            if isinstance(did, tuple):  # when separate=True
                 did = tuple([i.xreplace(undo) for i in did])
             elif did is not None:
                 did = did.xreplace(undo)
@@ -261,17 +274,23 @@ class Sum(AddWithLimits, ExprWithIntLimits):
     def eval_zeta_function(self, f, limits):
         """
         Check whether the function matches with the zeta function.
+
         If it matches, then return a `Piecewise` expression because
         zeta function does not converge unless `s > 1` and `q > 0`
         """
         i, a, b = limits
+        if a.is_comparable and b.is_comparable and a > b:
+            return self.eval_zeta_function(f, (i, b + S.One, a - S.One))
+        if b is not S.Infinity:
+            return
         w, y, z = Wild('w', exclude=[i]), Wild('y', exclude=[i]), Wild('z', exclude=[i])
-        result = f.match((w * i + y) ** (-z))
-        if result is not None and b is S.Infinity:
+        if result := f.match((w * i + y) ** (-z)):
             coeff = 1 / result[w] ** result[z]
             s = result[z]
             q = result[y] / result[w] + a
-            return Piecewise((coeff * zeta(s, q), And(q > 0, s > 1)), (self, True))
+            return Piecewise((coeff * zeta(s, q),
+                              And(Not(Contains(-q, S.Naturals0)), re(s) > S.One)),
+                             (self, True))
 
     def _eval_derivative(self, x):
         """
@@ -320,12 +339,14 @@ class Sum(AddWithLimits, ExprWithIntLimits):
         return Sum(f, (k, upper + 1, new_upper)).doit()
 
     def _eval_simplify(self, **kwargs):
-        from sympy.simplify.simplify import factor_sum, sum_combine
-        from sympy.core.function import expand
-        from sympy.core.mul import Mul
+
+        function = self.function
+
+        if kwargs.get('deep', True):
+            function = function.simplify(**kwargs)
 
         # split the function into adds
-        terms = Add.make_args(expand(self.function))
+        terms = Add.make_args(expand(function))
         s_t = [] # Sum Terms
         o_t = [] # Other Terms
 
@@ -340,7 +361,7 @@ class Sum(AddWithLimits, ExprWithIntLimits):
                     # go through each term
                     if isinstance(subterm, Sum):
                         # if it's a sum, simplify it
-                        out_terms.append(subterm._eval_simplify())
+                        out_terms.append(subterm._eval_simplify(**kwargs))
                     else:
                         # otherwise, add it as is
                         out_terms.append(subterm)
@@ -351,6 +372,7 @@ class Sum(AddWithLimits, ExprWithIntLimits):
                 o_t.append(term)
 
         # next try to combine any interior sums for further simplification
+        from sympy.simplify.simplify import factor_sum, sum_combine
         result = Add(sum_combine(s_t), *o_t)
 
         return factor_sum(result, limits=self.limits)
@@ -402,7 +424,7 @@ class Sum(AddWithLimits, ExprWithIntLimits):
         There are various tests employed to check the convergence like
         divergence test, root test, integral test, alternating series test,
         comparison tests, Dirichlet tests. It returns true if Sum is convergent
-        and false if divergent and NotImplementedError if it can not be checked.
+        and false if divergent and NotImplementedError if it cannot be checked.
 
         References
         ==========
@@ -426,10 +448,9 @@ class Sum(AddWithLimits, ExprWithIntLimits):
         See Also
         ========
 
-        Sum.is_absolutely_convergent()
-        sympy.concrete.products.Product.is_convergent()
+        Sum.is_absolutely_convergent
+        sympy.concrete.products.Product.is_convergent
         """
-        from sympy import Interval, Integral, log, symbols, simplify
         p, q, r = symbols('p q r', cls=Wild)
 
         sym = self.limits[0][0]
@@ -450,6 +471,7 @@ class Sum(AddWithLimits, ExprWithIntLimits):
             if upper_limit is S.Infinity:
                 return Sum(sequence_term, (sym, 0, S.Infinity)).is_convergent() and \
                         Sum(sequence_term, (sym, S.NegativeInfinity, 0)).is_convergent()
+            from sympy.simplify.simplify import simplify
             sequence_term = simplify(sequence_term.xreplace({sym: -sym}))
             lower_limit = -upper_limit
             upper_limit = S.Infinity
@@ -471,9 +493,9 @@ class Sum(AddWithLimits, ExprWithIntLimits):
 
         ###  -------- Divergence test ----------- ###
         try:
-           lim_val = limit_seq(sequence_term, sym)
-           if lim_val is not None and lim_val.is_zero is False:
-               return S.false
+            lim_val = limit_seq(sequence_term, sym)
+            if lim_val is not None and lim_val.is_zero is False:
+                return S.false
         except NotImplementedError:
             pass
 
@@ -496,7 +518,8 @@ class Sum(AddWithLimits, ExprWithIntLimits):
 
         ### ------------- comparison test ------------- ###
         # 1/(n**p*log(n)**q*log(log(n))**r) comparison
-        n_log_test = order.expr.match(1/(sym**p*log(sym)**q*log(log(sym))**r))
+        n_log_test = (order.expr.match(1/(sym**p*log(1/sym)**q*log(-log(1/sym))**r)) or
+                      order.expr.match(1/(sym**p*(-log(1/sym))**q*log(-log(1/sym))**r)))
         if n_log_test is not None:
             if (n_log_test[p] > 1 or
                 (n_log_test[p] == 1 and n_log_test[q] > 1) or
@@ -515,10 +538,12 @@ class Sum(AddWithLimits, ExprWithIntLimits):
 
         ### ----------- ratio test ---------------- ###
         next_sequence_term = sequence_term.xreplace({sym: sym + 1})
+        from sympy.simplify.combsimp import combsimp
+        from sympy.simplify.powsimp import powsimp
         ratio = combsimp(powsimp(next_sequence_term/sequence_term))
         try:
             lim_ratio = limit_seq(ratio, sym)
-            if lim_ratio is not None and lim_ratio.is_number:
+            if lim_ratio is not None and lim_ratio.is_number and lim_ratio is not S.NaN:
                 if abs(lim_ratio) > 1:
                     return S.false
                 if abs(lim_ratio) < 1:
@@ -554,12 +579,13 @@ class Sum(AddWithLimits, ExprWithIntLimits):
             pass
 
         ### ------------- alternating series test ----------- ###
-        dict_val = sequence_term.match((-1)**(sym + p)*q)
+        dict_val = sequence_term.match(S.NegativeOne**(sym + p)*q)
         if not dict_val[p].has(sym) and is_decreasing(dict_val[q], interval):
             return S.true
 
         ### ------------- integral test -------------- ###
         check_interval = None
+        from sympy.solvers.solveset import solveset
         maxima = solveset(sequence_term.diff(sym), sym, interval)
         if not maxima:
             check_interval = interval
@@ -668,7 +694,7 @@ class Sum(AddWithLimits, ExprWithIntLimits):
         See Also
         ========
 
-        Sum.is_convergent()
+        Sum.is_convergent
         """
         return Sum(abs(self.function), self.limits).is_convergent()
 
@@ -715,9 +741,6 @@ class Sum(AddWithLimits, ExprWithIntLimits):
         With a nonzero eps specified, the summation is ended
         as soon as the remainder term is less than the epsilon.
         """
-        from sympy.functions import bernoulli, factorial
-        from sympy.integrals import Integral
-
         m = int(m)
         n = int(n)
         f = self.function
@@ -734,8 +757,7 @@ class Sum(AddWithLimits, ExprWithIntLimits):
             if b.is_Integer and a.is_Integer:
                 m = min(m, b - a + 1)
             if not eps or f.is_polynomial(i):
-                for k in range(m):
-                    s += f.subs(i, a + k)
+                s = Add(*[f.subs(i, a + k) for k in range(m)])
             else:
                 term = f.subs(i, a)
                 if term:
@@ -745,7 +767,7 @@ class Sum(AddWithLimits, ExprWithIntLimits):
                     elif not (test == False):
                         # a symbolic Relational class, can't go further
                         return term, S.Zero
-                s += term
+                s = term
                 for k in range(1, m):
                     term = f.subs(i, a + k)
                     if abs(term.evalf(3)) < eps and term != 0:
@@ -770,8 +792,14 @@ class Sum(AddWithLimits, ExprWithIntLimits):
         for k in range(1, n + 2):
             ga, gb = fpoint(g)
             term = bernoulli(2*k)/factorial(2*k)*(gb - ga)
-            if (eps and term and abs(term.evalf(3)) < eps) or (k > n):
+            if k > n:
                 break
+            if eps and term:
+                term_evalf = term.evalf(3)
+                if term_evalf is S.NaN:
+                    return S.NaN, S.NaN
+                if abs(term_evalf) < eps:
+                    break
             s += term
             g = g.diff(i, 2, simplify=False)
         return s + iterm, abs(term)
@@ -837,7 +865,7 @@ class Sum(AddWithLimits, ExprWithIntLimits):
 
         .. [1] Michael Karr, "Summation in Finite Terms", Journal of the ACM,
                Volume 28 Issue 2, April 1981, Pages 305-350
-               http://dl.acm.org/citation.cfm?doid=322248.322255
+               https://dl.acm.org/doi/10.1145/322248.322255
         """
         l_indices = list(indices)
 
@@ -856,6 +884,11 @@ class Sum(AddWithLimits, ExprWithIntLimits):
 
         return Sum(e * self.function, *limits)
 
+    def _eval_rewrite_as_Product(self, *args, **kwargs):
+        from sympy.concrete.products import Product
+        if self.function.is_extended_real:
+            return log(Product(exp(self.function), *self.limits))
+
 
 def summation(f, *symbols, **kwargs):
     r"""
@@ -931,10 +964,7 @@ def telescopic_direct(L, R, n, limits):
 
     """
     (i, a, b) = limits
-    s = 0
-    for m in range(n):
-        s += L.subs(i, a + m) + R.subs(i, b - m)
-    return s
+    return Add(*[L.subs(i, a + m) + R.subs(i, b - m) for m in range(n)])
 
 
 def telescopic(L, R, limits):
@@ -949,15 +979,16 @@ def telescopic(L, R, limits):
 
     # We want to solve(L.subs(i, i + m) + R, m)
     # First we try a simple match since this does things that
-    # solve doesn't do, e.g. solve(f(k+m)-f(k), m) fails
+    # solve doesn't do, e.g. solve(cos(k+m)-cos(k), m) gives
+    # a more complicated solution than m == 0.
 
     k = Wild("k")
     sol = (-R).match(L.subs(i, i + k))
     s = None
     if sol and k in sol:
         s = sol[k]
-        if not (s.is_Integer and L.subs(i, i + s) == -R):
-            # sometimes match fail(f(x+2).match(-f(x+k))->{k: -2 - 2x}))
+        if not (s.is_Integer and L.subs(i, i + s) + R == 0):
+            # invalid match or match didn't work
             s = None
 
     # But there are things that match doesn't do that solve
@@ -966,6 +997,7 @@ def telescopic(L, R, limits):
     if s is None:
         m = Dummy('m')
         try:
+            from sympy.solvers.solvers import solve
             sol = solve(L.subs(i, i + m) + R, m) or []
         except NotImplementedError:
             return None
@@ -982,9 +1014,6 @@ def telescopic(L, R, limits):
 
 
 def eval_sum(f, limits):
-    from sympy.concrete.delta import deltasummation, _has_simple_delta
-    from sympy.functions import KroneckerDelta
-
     (i, a, b) = limits
     if f.is_zero:
         return S.Zero
@@ -992,6 +1021,8 @@ def eval_sum(f, limits):
         return f*(b - a + 1)
     if a == b:
         return f.subs(i, a)
+    if a.is_comparable and b.is_comparable and a > b:
+        return eval_sum(f, (i, b + S.One, a - S.One))
     if isinstance(f, Piecewise):
         if not any(i in arg.args[1].free_symbols for arg in f.args):
             # Piecewise conditions do not depend on the dummy summation variable,
@@ -1006,6 +1037,7 @@ def eval_sum(f, limits):
             return f.func(*newargs)
 
     if f.has(KroneckerDelta):
+        from .delta import deltasummation, _has_simple_delta
         f = f.replace(
             lambda x: isinstance(x, Sum),
             lambda x: x.factor()
@@ -1020,9 +1052,8 @@ def eval_sum(f, limits):
         return eval_sum_direct(f, (i, a, b))
     if isinstance(f, Piecewise):
         return None
-    # Try to do it symbolically. Even when the number of terms is known,
-    # this can save time when b-a is big.
-    # We should try to transform to partial fractions
+    # Try to do it symbolically. Even when the number of terms is
+    # known, this can save time when b-a is big.
     value = eval_sum_symbolic(f.expand(), (i, a, b))
     if value is not None:
         return value
@@ -1036,7 +1067,6 @@ def eval_sum_direct(expr, limits):
     Evaluate expression directly, but perform some simple checks first
     to possibly result in a smaller expression and faster execution.
     """
-    from sympy.core import Add
     (i, a, b) = limits
 
     dif = b - a
@@ -1063,10 +1093,15 @@ def eval_sum_direct(expr, limits):
                 sL = eval_sum_direct(L, (i, a, b))
                 if sL:
                     return sL*R
-        try:
-            expr = apart(expr, i)  # see if it becomes an Add
-        except PolynomialError:
-            pass
+
+    # do this whether its an Add or Mul
+    # e.g. apart(1/(25*i**2 + 45*i + 14)) and
+    # apart(1/((5*i + 2)*(5*i + 7))) ->
+    # -1/(5*(5*i + 7)) + 1/(5*(5*i + 2))
+    try:
+        expr = apart(expr, i)  # see if it becomes an Add
+    except PolynomialError:
+        pass
 
     if expr.is_Add:
         # Try factor out everything not including i
@@ -1092,8 +1127,6 @@ def eval_sum_direct(expr, limits):
 
 
 def eval_sum_symbolic(f, limits):
-    from sympy.functions import harmonic, bernoulli
-
     f_orig = f
     (i, a, b) = limits
     if not f.has(i):
@@ -1122,10 +1155,15 @@ def eval_sum_symbolic(f, limits):
                 sL = eval_sum_symbolic(L, (i, a, b))
                 if sL:
                     return sL*R
-        try:
-            f = apart(f, i)  # see if it becomes an Add
-        except PolynomialError:
-            pass
+
+    # do this whether its an Add or Mul
+    # e.g. apart(1/(25*i**2 + 45*i + 14)) and
+    # apart(1/((5*i + 2)*(5*i + 7))) ->
+    # -1/(5*(5*i + 7)) + 1/(5*(5*i + 2))
+    try:
+        f = apart(f, i)
+    except PolynomialError:
+        pass
 
     if f.is_Add:
         L, R = f.as_two_terms()
@@ -1162,8 +1200,8 @@ def eval_sum_symbolic(f, limits):
 
         if n.is_Integer:
             if n >= 0:
-                if (b is S.Infinity and not a is S.NegativeInfinity) or \
-                   (a is S.NegativeInfinity and not b is S.Infinity):
+                if (b is S.Infinity and a is not S.NegativeInfinity) or \
+                   (a is S.NegativeInfinity and b is not S.Infinity):
                     return S.Infinity
                 return ((bernoulli(n + 1, b + 1) - bernoulli(n + 1, a))/(n + 1)).expand()
             elif a.is_Integer and a >= 1:
@@ -1198,7 +1236,8 @@ def eval_sum_symbolic(f, limits):
         r = gosper_sum(f, (i, a, b))
 
         if isinstance(r, (Mul,Add)):
-            from sympy import ordered, Tuple
+            from sympy.simplify.radsimp import denom
+            from sympy.solvers.solvers import solve
             non_limit = r.free_symbols - Tuple(*limits[1:]).free_symbols
             den = denom(together(r))
             den_sym = non_limit & den.free_symbols
@@ -1216,7 +1255,7 @@ def eval_sum_symbolic(f, limits):
             args.append((r, True))
             return Piecewise(*args)
 
-        if not r in (None, S.NaN):
+        if r not in (None, S.NaN):
             return r
 
     h = eval_sum_hyper(f_orig, (i, a, b))
@@ -1234,19 +1273,16 @@ def eval_sum_symbolic(f, limits):
 
 def _eval_sum_hyper(f, i, a):
     """ Returns (res, cond). Sums from a to oo. """
-    from sympy.functions import hyper
-    from sympy.simplify import hyperexpand, hypersimp, fraction, simplify
-    from sympy.polys.polytools import Poly, factor
-    from sympy.core.numbers import Float
-
     if a != 0:
         return _eval_sum_hyper(f.subs(i, i + a), i, 0)
 
     if f.subs(i, 0) == 0:
+        from sympy.simplify.simplify import simplify
         if simplify(f.subs(i, Dummy('i', integer=True, positive=True))) == 0:
             return S.Zero, True
         return _eval_sum_hyper(f.subs(i, i + 1), i, 0)
 
+    from sympy.simplify.simplify import hypersimp
     hs = hypersimp(f, i)
     if hs is None:
         return None
@@ -1255,6 +1291,9 @@ def _eval_sum_hyper(f, i, a):
         from sympy.simplify.simplify import nsimplify
         hs = nsimplify(hs)
 
+    from sympy.simplify.combsimp import combsimp
+    from sympy.simplify.hyperexpand import hyperexpand
+    from sympy.simplify.radsimp import fraction
     numer, denom = fraction(factor(hs))
     top, topl = numer.as_coeff_mul(i)
     bot, botl = denom.as_coeff_mul(i)
@@ -1287,10 +1326,11 @@ def _eval_sum_hyper(f, i, a):
 
 
 def eval_sum_hyper(f, i_a_b):
-    from sympy.logic.boolalg import And
-
     i, a, b = i_a_b
 
+    if f.is_hypergeometric(i) is False:
+        return
+
     if (b - a).is_Integer:
         # We are never going to do better than doing the sum in the obvious way
         return None
@@ -1303,10 +1343,15 @@ def eval_sum_hyper(f, i_a_b):
             if res is not None:
                 return Piecewise(res, (old_sum, True))
         else:
+            n_illegal = lambda x: sum(x.count(_) for _ in _illegal)
+            had = n_illegal(f)
+            # check that no extra illegals are introduced
             res1 = _eval_sum_hyper(f, i, a)
+            if res1 is None or n_illegal(res1) > had:
+                return
             res2 = _eval_sum_hyper(f, i, b + 1)
-            if res1 is None or res2 is None:
-                return None
+            if res2 is None or n_illegal(res2) > had:
+                return
             (res1, cond1), (res2, cond2) = res1, res2
             cond = And(cond1, cond2)
             if cond == False:
@@ -1346,7 +1391,7 @@ def eval_sum_residue(f, i_a_b):
     Notes
     =====
 
-    If $f(n), g(n)$ are polynomials with $\deg(g(n)) - \deg(f(n)) >= 2$,
+    If $f(n), g(n)$ are polynomials with $\deg(g(n)) - \deg(f(n)) \ge 2$,
     some infinite summations can be computed by the following residue
     evaluations.
 
@@ -1411,12 +1456,16 @@ def eval_sum_residue(f, i_a_b):
     """
     i, a, b = i_a_b
 
+    # If lower limit > upper limit: Karr Summation Convention
+    if a.is_comparable and b.is_comparable and a > b:
+        return eval_sum_residue(f, (i, b + S.One, a - S.One))
+
     def is_even_function(numer, denom):
         """Test if the rational function is an even function"""
-        numer_even = all([i % 2 == 0 for (i,) in numer.monoms()])
-        denom_even = all([i % 2 == 0 for (i,) in denom.monoms()])
-        numer_odd = all([i % 2 == 1 for (i,) in numer.monoms()])
-        denom_odd = all([i % 2 == 1 for (i,) in denom.monoms()])
+        numer_even = all(i % 2 == 0 for (i,) in numer.monoms())
+        denom_even = all(i % 2 == 0 for (i,) in denom.monoms())
+        numer_odd = all(i % 2 == 1 for (i,) in numer.monoms())
+        denom_odd = all(i % 2 == 1 for (i,) in denom.monoms())
         return (numer_even and denom_even) or (numer_odd and denom_odd)
 
     def match_rational(f, i):
@@ -1442,15 +1491,19 @@ def eval_sum_residue(f, i_a_b):
         shift = - b / a / n
         return shift
 
+    #Need a dummy symbol with no assumptions set for get_residue_factor
+    z = Dummy('z')
+
     def get_residue_factor(numer, denom, alternating):
+        residue_factor = (numer.as_expr() / denom.as_expr()).subs(i, z)
         if not alternating:
-            residue_factor = (numer.as_expr() / denom.as_expr()) * cot(S.Pi * i)
+            residue_factor *= cot(S.Pi * z)
         else:
-            residue_factor = (numer.as_expr() / denom.as_expr()) * csc(S.Pi * i)
+            residue_factor *= csc(S.Pi * z)
         return residue_factor
 
     # We don't know how to deal with symbolic constants in summand
-    if f.free_symbols - set([i]):
+    if f.free_symbols - {i}:
         return None
 
     if not (a.is_Integer or a in (S.Infinity, S.NegativeInfinity)):
@@ -1467,7 +1520,7 @@ def eval_sum_residue(f, i_a_b):
         alternating = False
         numer, denom = match
     else:
-        match = match_rational(f / (-1)**i, i)
+        match = match_rational(f / S.NegativeOne**i, i)
         if match:
             alternating = True
             numer, denom = match
@@ -1487,7 +1540,7 @@ def eval_sum_residue(f, i_a_b):
             return None
 
         residue_factor = get_residue_factor(numer, denom, alternating)
-        residues = [residue(residue_factor, i, root) for root in nonint_roots]
+        residues = [residue(residue_factor, z, root) for root in nonint_roots]
         return -S.Pi * sum(residues)
 
     if not (a.is_finite and b is S.Infinity):
@@ -1511,7 +1564,7 @@ def eval_sum_residue(f, i_a_b):
             return None
 
         if alternating:
-            f = (-1)**i * ((-1)**shift * numer.as_expr() / denom.as_expr())
+            f = S.NegativeOne**i * (S.NegativeOne**shift * numer.as_expr() / denom.as_expr())
         else:
             f = numer.as_expr() / denom.as_expr()
         return eval_sum_residue(f, (i, a-shift, b-shift))
@@ -1535,7 +1588,7 @@ def eval_sum_residue(f, i_a_b):
             return None
 
     residue_factor = get_residue_factor(numer, denom, alternating)
-    residues = [residue(residue_factor, i, root) for root in int_roots + nonint_roots]
+    residues = [residue(residue_factor, z, root) for root in int_roots + nonint_roots]
     full_sum = -S.Pi * sum(residues)
 
     if not int_roots:
@@ -1562,7 +1615,7 @@ def eval_sum_residue(f, i_a_b):
 
 def _eval_matrix_sum(expression):
     f = expression.function
-    for n, limit in enumerate(expression.limits):
+    for limit in expression.limits:
         i, a, b = limit
         dif = b - a
         if dif.is_Integer:
@@ -1599,7 +1652,7 @@ def _dummy_with_inherited_properties_concrete(limits):
         assum_true = x._assumptions.get(assum, None)
         if assum_true:
             assumptions_to_keep[assum] = True
-        elif all([getattr(i, 'is_' + assum) for i in l]):
+        elif all(getattr(i, 'is_' + assum) for i in l):
             assumptions_to_add[assum] = True
     if assumptions_to_add:
         assumptions_to_keep.update(assumptions_to_add)
