@@ -1277,10 +1277,10 @@ def test_issue_8919():
     ans = I.doit()
     assert ans == Piecewise((0, b > c), (a*Min(y, z) - a*Min(0, z), True))
     for cond in (True, False):
-         for yy in range(1, 3):
-             for zz in range(-yy, 0, yy):
-                 reps = [(b > c, cond), (y, yy), (z, zz)]
-                 assert ans.subs(reps) == I.subs(reps).doit()
+        for yy in range(1, 3):
+            for zz in range(-yy, 0, yy):
+                reps = [(b > c, cond), (y, yy), (z, zz)]
+                assert ans.subs(reps) == I.subs(reps).doit()
 
 
 def test_unevaluated_integrals():
@@ -1335,6 +1335,17 @@ def test_Piecewise_rewrite_as_ITE():
     raises(ValueError, lambda: _ITE((a, x < 2), (b, x > 3)))
 
 
+def test_Piecewise_replace_relational_27538():
+    x, y = symbols('x, y')
+    p1 = Piecewise(
+        (0, Eq(x, True)),
+        (1, True),
+    )
+    p2 = p1.xreplace({x: y < 1})
+    assert p2.subs(y, 0) == 0
+    assert p2.subs(y, 1) == 1
+
+
 def test_issue_14052():
     assert integrate(abs(sin(x)), (x, 0, 2*pi)) == 4
 
