@@ -1,7 +1,11 @@
-from sympy import Abs, S, Symbol, symbols, I, Rational, PurePoly, Float
+from sympy.core.numbers import (Float, I, Rational)
+from sympy.core.singleton import S
+from sympy.core.symbol import (Symbol, symbols)
+from sympy.functions.elementary.complexes import Abs
+from sympy.polys.polytools import PurePoly
 from sympy.matrices import \
     Matrix, MutableSparseMatrix, ImmutableSparseMatrix, SparseMatrix, eye, \
-    ones, zeros, ShapeError
+    ones, zeros, ShapeError, NonSquareMatrixError
 from sympy.testing.pytest import raises
 
 
@@ -30,7 +34,7 @@ def test_sparse_matrix():
     ))
     assert SparseMatrix(a) == a
 
-    from sympy.matrices import MutableSparseMatrix, MutableDenseMatrix
+    from sympy.matrices import MutableDenseMatrix
     a = MutableSparseMatrix([])
     b = MutableDenseMatrix([1, 2])
     assert a.row_join(b) == b
@@ -626,7 +630,6 @@ def test_copyin():
 
 
 def test_sparse_solve():
-    from sympy.matrices import SparseMatrix
     A = SparseMatrix(((25, 15, -5), (15, 18, 0), (-5, 0, 11)))
     assert A.cholesky() == Matrix([
         [ 5, 0, 0],
@@ -676,6 +679,13 @@ def test_sparse_solve():
 
 
 def test_lower_triangular_solve():
+    raises(NonSquareMatrixError, lambda:
+        SparseMatrix([[1, 2]]).lower_triangular_solve(Matrix([[1, 2]])))
+    raises(ShapeError, lambda:
+        SparseMatrix([[1, 2], [0, 4]]).lower_triangular_solve(Matrix([1])))
+    raises(ValueError, lambda:
+        SparseMatrix([[1, 2], [3, 4]]).lower_triangular_solve(Matrix([[1, 2], [3, 4]])))
+
     a, b, c, d = symbols('a:d')
     u, v, w, x = symbols('u:x')
 
@@ -689,6 +699,13 @@ def test_lower_triangular_solve():
 
 
 def test_upper_triangular_solve():
+    raises(NonSquareMatrixError, lambda:
+        SparseMatrix([[1, 2]]).upper_triangular_solve(Matrix([[1, 2]])))
+    raises(ShapeError, lambda:
+        SparseMatrix([[1, 2], [0, 4]]).upper_triangular_solve(Matrix([1])))
+    raises(TypeError, lambda:
+        SparseMatrix([[1, 2], [3, 4]]).upper_triangular_solve(Matrix([[1, 2], [3, 4]])))
+
     a, b, c, d = symbols('a:d')
     u, v, w, x = symbols('u:x')
 
