@@ -1,16 +1,17 @@
-from .lti import (TransferFunction, Series, MIMOSeries, Parallel, MIMOParallel,
+from .lti import (TransferFunction, PIDController, Series, MIMOSeries, Parallel, MIMOParallel,
     Feedback, MIMOFeedback, TransferFunctionMatrix, StateSpace, gbt, bilinear, forward_diff,
     backward_diff, phase_margin, gain_margin)
 from .control_plots import (pole_zero_numerical_data, pole_zero_plot, step_response_numerical_data,
     step_response_plot, impulse_response_numerical_data, impulse_response_plot, ramp_response_numerical_data,
     ramp_response_plot, bode_magnitude_numerical_data, bode_phase_numerical_data, bode_magnitude_plot,
-    bode_phase_plot, bode_plot)
+    bode_phase_plot, bode_plot, nyquist_plot_expr, nyquist_plot, nichols_plot_expr, nichols_plot)
 
-__all__ = ['TransferFunction', 'Series', 'MIMOSeries', 'Parallel',
+__all__ = ['TransferFunction', 'PIDController', 'Series', 'MIMOSeries', 'Parallel',
     'MIMOParallel', 'Feedback', 'MIMOFeedback', 'TransferFunctionMatrix', 'StateSpace',
     'gbt', 'bilinear', 'forward_diff', 'backward_diff', 'phase_margin', 'gain_margin',
     'pole_zero_numerical_data', 'pole_zero_plot', 'step_response_numerical_data',
     'step_response_plot', 'impulse_response_numerical_data', 'impulse_response_plot',
     'ramp_response_numerical_data', 'ramp_response_plot',
     'bode_magnitude_numerical_data', 'bode_phase_numerical_data',
-    'bode_magnitude_plot', 'bode_phase_plot', 'bode_plot']
+    'bode_magnitude_plot', 'bode_phase_plot', 'bode_plot', 'nyquist_plot_expr', 'nyquist_plot',
+    'nichols_plot_expr', 'nichols_plot']
