@@ -25,12 +25,19 @@ __all__ = ['refraction_angle',
            'transverse_magnification'
            ]
 
-from sympy import Symbol, sympify, sqrt, Matrix, acos, oo, Limit, atan2, asin,\
-cos, sin, tan, I, cancel, pi, Float
-from sympy.core.compatibility import is_sequence
+from sympy.core.numbers import (Float, I, oo, pi, zoo)
+from sympy.core.singleton import S
+from sympy.core.symbol import Symbol
+from sympy.core.sympify import sympify
+from sympy.functions.elementary.miscellaneous import sqrt
+from sympy.functions.elementary.trigonometric import (acos, asin, atan2, cos, sin, tan)
+from sympy.matrices.dense import Matrix
+from sympy.polys.polytools import cancel
+from sympy.series.limits import Limit
 from sympy.geometry.line import Ray3D
 from sympy.geometry.util import intersection
 from sympy.geometry.plane import Plane
+from sympy.utilities.iterables import is_sequence
 from .medium import Medium
 
 
@@ -189,8 +196,8 @@ def refraction_angle(incident, medium1, medium2, normal=None, plane=None):
 
     eta = n1/n2  # Relative index of refraction
     # Calculating magnitude of the vectors
-    mag_incident = sqrt(sum([i**2 for i in _incident]))
-    mag_normal = sqrt(sum([i**2 for i in _normal]))
+    mag_incident = sqrt(sum(i**2 for i in _incident))
+    mag_normal = sqrt(sum(i**2 for i in _normal))
     # Converting vectors to unit vectors by dividing
     # them with their magnitudes
     _incident /= mag_incident
@@ -198,7 +205,7 @@ def refraction_angle(incident, medium1, medium2, normal=None, plane=None):
     c1 = -_incident.dot(_normal)  # cos(angle_of_incidence)
     cs2 = 1 - eta**2*(1 - c1**2)  # cos(angle_of_refraction)**2
     if cs2.is_negative:  # This is the case of total internal reflection(TIR).
-        return 0
+        return S.Zero
     drs = eta*_incident + (eta*c1 - sqrt(cs2))*_normal
     # Multiplying unit vector by its magnitude
     drs = drs*mag_incident
@@ -375,9 +382,9 @@ def deviation(incident, medium1, medium2, normal=None, plane=None):
         else:
             _normal = Matrix(plane.normal_vector)
 
-        mag_incident = sqrt(sum([i**2 for i in _incident]))
-        mag_normal = sqrt(sum([i**2 for i in _normal]))
-        mag_refracted = sqrt(sum([i**2 for i in refracted]))
+        mag_incident = sqrt(sum(i**2 for i in _incident))
+        mag_normal = sqrt(sum(i**2 for i in _normal))
+        mag_refracted = sqrt(sum(i**2 for i in refracted))
         _incident /= mag_incident
         _normal /= mag_normal
         refracted /= mag_refracted
@@ -445,9 +452,9 @@ def critical_angle(medium1, medium2):
 
 
 
-def lens_makers_formula(n_lens, n_surr, r1, r2):
+def lens_makers_formula(n_lens, n_surr, r1, r2, d=0):
     """
-    This function calculates focal length of a thin lens.
+    This function calculates focal length of a lens.
     It follows cartesian sign convention.
 
     Parameters
@@ -461,15 +468,23 @@ def lens_makers_formula(n_lens, n_surr, r1, r2):
         Radius of curvature of first surface.
     r2 : sympifiable
         Radius of curvature of second surface.
+    d : sympifiable, optional
+        Thickness of lens, default value is 0.
 
     Examples
     ========
 
     >>> from sympy.physics.optics import lens_makers_formula
+    >>> from sympy import S
     >>> lens_makers_formula(1.33, 1, 10, -10)
     15.1515151515151
+    >>> lens_makers_formula(1.2, 1, 10, S.Infinity)
+    50.0000000000000
+    >>> lens_makers_formula(1.33, 1, 10, -10, d=1)
+    15.3418463277618
 
     """
+
     if isinstance(n_lens, Medium):
         n_lens = n_lens.refractive_index
     else:
@@ -478,11 +493,13 @@ def lens_makers_formula(n_lens, n_surr, r1, r2):
         n_surr = n_surr.refractive_index
     else:
         n_surr = sympify(n_surr)
+    d = sympify(d)
 
-    r1 = sympify(r1)
-    r2 = sympify(r2)
+    focal_length = 1/((n_lens - n_surr) / n_surr*(1/r1 - 1/r2 + (((n_lens - n_surr) * d) / (n_lens * r1 * r2))))
 
-    return 1/((n_lens - n_surr)/n_surr*(1/r1 - 1/r2))
+    if focal_length == zoo:
+        return S.Infinity
+    return focal_length
 
 
 def mirror_formula(focal_length=None, u=None, v=None):
@@ -654,8 +671,8 @@ def hyperfocal_distance(f, N, c):
 def transverse_magnification(si, so):
     """
 
-    Calculates the transverse magnification, which is the ratio of the
-    image size to the object size.
+    Calculates the transverse magnification upon reflection in a mirror,
+    which is the ratio of the image size to the object size.
 
     Parameters
     ==========
