@@ -1,4 +1,6 @@
-from sympy import Symbol, sympify
+from sympy.core.singleton import S
+from sympy.core.symbol import Symbol
+from sympy.core.sympify import sympify
 from sympy.core.numbers import Integer
 
 
@@ -53,7 +55,7 @@ class PlotInterval:
             self._v = None
             return
         if not isinstance(v, Symbol):
-            raise ValueError("v must be a sympy Symbol.")
+            raise ValueError("v must be a SymPy Symbol.")
         self._v = v
 
     def get_v_min(self):
@@ -92,8 +94,8 @@ class PlotInterval:
         if isinstance(v_steps, int):
             v_steps = Integer(v_steps)
         elif not isinstance(v_steps, Integer):
-            raise ValueError("v_steps must be an int or sympy Integer.")
-        if v_steps <= Integer(0):
+            raise ValueError("v_steps must be an int or SymPy Integer.")
+        if v_steps <= S.Zero:
             raise ValueError("v_steps must be positive.")
         self._v_steps = v_steps
 
@@ -153,7 +155,7 @@ class PlotInterval:
     @require_all_args
     def vrange(self):
         """
-        Yields v_steps+1 sympy numbers ranging from
+        Yields v_steps+1 SymPy numbers ranging from
         v_min to v_max.
         """
         d = (self.v_max - self.v_min) / self.v_steps
@@ -164,11 +166,11 @@ class PlotInterval:
     @require_all_args
     def vrange2(self):
         """
-        Yields v_steps pairs of sympy numbers ranging from
+        Yields v_steps pairs of SymPy numbers ranging from
         (v_min, v_min + step) to (v_max - step, v_max).
         """
         d = (self.v_max - self.v_min) / self.v_steps
-        a = self.v_min + (d * Integer(0))
+        a = self.v_min + (d * S.Zero)
         for i in range(self.v_steps):
             b = self.v_min + (d * Integer(i + 1))
             yield a, b
