@@ -4,47 +4,46 @@
 from itertools import combinations_with_replacement, product
 from textwrap import dedent
 
+from sympy.core.cache import cacheit
 from sympy.core import Mul, S, Tuple, sympify
-from sympy.core.compatibility import exec_, iterable
 from sympy.polys.polyerrors import ExactQuotientFailed
 from sympy.polys.polyutils import PicklableWithSlots, dict_from_expr
 from sympy.utilities import public
-from sympy.core.compatibility import is_sequence
+from sympy.utilities.iterables import is_sequence, iterable
 
 @public
 def itermonomials(variables, max_degrees, min_degrees=None):
     r"""
-    `max_degrees` and `min_degrees` are either both integers or both lists.
-    Unless otherwise specified, `min_degrees` is either 0 or [0,...,0].
+    ``max_degrees`` and ``min_degrees`` are either both integers or both lists.
+    Unless otherwise specified, ``min_degrees`` is either ``0`` or
+    ``[0, ..., 0]``.
 
-    A generator of all monomials `monom` is returned, such that
+    A generator of all monomials ``monom`` is returned, such that
     either
-    min_degree <= total_degree(monom) <= max_degree,
+    ``min_degree <= total_degree(monom) <= max_degree``,
     or
-    min_degrees[i] <= degree_list(monom)[i] <= max_degrees[i], for all i.
+    ``min_degrees[i] <= degree_list(monom)[i] <= max_degrees[i]``,
+    for all ``i``.
 
-    Case I:: `max_degrees` and `min_degrees` are both integers.
-    ===========================================================
-    Given a set of variables `V` and a min_degree `N` and a max_degree `M`
-    generate a set of monomials of degree less than or equal to `N` and greater
-    than or equal to `M`. The total number of monomials in commutative
-    variables is huge and is given by the following formula if `M = 0`:
+    Case I. ``max_degrees`` and ``min_degrees`` are both integers
+    =============================================================
+
+    Given a set of variables $V$ and a min_degree $N$ and a max_degree $M$
+    generate a set of monomials of degree less than or equal to $N$ and greater
+    than or equal to $M$. The total number of monomials in commutative
+    variables is huge and is given by the following formula if $M = 0$:
 
         .. math::
-
             \frac{(\#V + N)!}{\#V! N!}
 
     For example if we would like to generate a dense polynomial of
-    a total degree `N = 50` and `M = 0`, which is the worst case, in 5
+    a total degree $N = 50$ and $M = 0$, which is the worst case, in 5
     variables, assuming that exponents and all of coefficients are 32-bit long
     and stored in an array we would need almost 80 GiB of memory! Fortunately
     most polynomials, that we will encounter, are sparse.
 
-    Examples
-    ========
-
-    Consider monomials in commutative variables `x` and `y`
-    and non-commutative variables `a` and `b`::
+    Consider monomials in commutative variables $x$ and $y$
+    and non-commutative variables $a$ and $b$::
 
         >>> from sympy import symbols
         >>> from sympy.polys.monomials import itermonomials
@@ -64,19 +63,19 @@ def itermonomials(variables, max_degrees, min_degrees=None):
         >>> sorted(itermonomials([x, y], 2, 1), key=monomial_key('grlex', [y, x]))
         [x, y, x**2, x*y, y**2]
 
+    Case II. ``max_degrees`` and ``min_degrees`` are both lists
+    ===========================================================
 
-    Case II:: `max_degrees` and `min_degrees` are both lists.
-    =========================================================
-    If max_degrees = [d_1, ..., d_n] and min_degrees = [e_1, ..., e_n],
-    the number of monomials generated is:
+    If ``max_degrees = [d_1, ..., d_n]`` and
+    ``min_degrees = [e_1, ..., e_n]``, the number of monomials generated
+    is:
 
-        (d_1 - e_1 + 1) * ... * (d_n - e_n + 1)
+    .. math::
+        (d_1 - e_1 + 1) (d_2 - e_2 + 1) \cdots (d_n - e_n + 1)
 
-    Example
-    =======
-
-    Let us generate all monomials `monom` in variables `x`, and `y`
-    such that [1, 2][i] <= degree_list(monom)[i] <= [2, 4][i], i = 0, 1 ::
+    Let us generate all monomials ``monom`` in variables $x$ and $y$
+    such that ``[1, 2][i] <= degree_list(monom)[i] <= [2, 4][i]``,
+    ``i = 0, 1`` ::
 
         >>> from sympy import symbols
         >>> from sympy.polys.monomials import itermonomials
@@ -86,8 +85,8 @@ def itermonomials(variables, max_degrees, min_degrees=None):
         >>> sorted(itermonomials([x, y], [2, 4], [1, 2]), reverse=True, key=monomial_key('lex', [x, y]))
         [x**2*y**4, x**2*y**3, x**2*y**2, x*y**4, x*y**3, x*y**2]
     """
-    n = len(variables)
     if is_sequence(max_degrees):
+        n = len(variables)
         if len(max_degrees) != n:
             raise ValueError('Argument sizes do not match')
         if min_degrees is None:
@@ -98,20 +97,24 @@ def itermonomials(variables, max_degrees, min_degrees=None):
             if len(min_degrees) != n:
                 raise ValueError('Argument sizes do not match')
             if any(i < 0 for i in min_degrees):
-                raise ValueError("min_degrees can't contain negative numbers")
-        total_degree = False
+                raise ValueError("min_degrees cannot contain negative numbers")
+        if any(min_degrees[i] > max_degrees[i] for i in range(n)):
+            raise ValueError('min_degrees[i] must be <= max_degrees[i] for all i')
+        power_lists = []
+        for var, min_d, max_d in zip(variables, min_degrees, max_degrees):
+            power_lists.append([var**i for i in range(min_d, max_d + 1)])
+        for powers in product(*power_lists):
+            yield Mul(*powers)
     else:
         max_degree = max_degrees
         if max_degree < 0:
-            raise ValueError("max_degrees can't be negative")
+            raise ValueError("max_degrees cannot be negative")
         if min_degrees is None:
             min_degree = 0
         else:
             if min_degrees < 0:
-                raise ValueError("min_degrees can't be negative")
+                raise ValueError("min_degrees cannot be negative")
             min_degree = min_degrees
-        total_degree = True
-    if total_degree:
         if min_degree > max_degree:
             return
         if not variables or max_degree == 0:
@@ -120,37 +123,21 @@ def itermonomials(variables, max_degrees, min_degrees=None):
         # Force to list in case of passed tuple or other incompatible collection
         variables = list(variables) + [S.One]
         if all(variable.is_commutative for variable in variables):
-            monomials_list_comm = []
-            for item in combinations_with_replacement(variables, max_degree):
-                powers = dict()
-                for variable in variables:
-                    powers[variable] = 0
-                for variable in item:
-                    if variable != 1:
-                        powers[variable] += 1
-                if max(powers.values()) >= min_degree:
-                    monomials_list_comm.append(Mul(*item))
-            yield from set(monomials_list_comm)
+            it = combinations_with_replacement(variables, max_degree)
         else:
-            monomials_list_non_comm = []
-            for item in product(variables, repeat=max_degree):
-                powers = dict()
-                for variable in variables:
-                    powers[variable] = 0
-                for variable in item:
-                    if variable != 1:
-                        powers[variable] += 1
-                if max(powers.values()) >= min_degree:
-                    monomials_list_non_comm.append(Mul(*item))
-            yield from set(monomials_list_non_comm)
-    else:
-        if any(min_degrees[i] > max_degrees[i] for i in range(n)):
-            raise ValueError('min_degrees[i] must be <= max_degrees[i] for all i')
-        power_lists = []
-        for var, min_d, max_d in zip(variables, min_degrees, max_degrees):
-            power_lists.append([var**i for i in range(min_d, max_d + 1)])
-        for powers in product(*power_lists):
-            yield Mul(*powers)
+            it = product(variables, repeat=max_degree)
+        monomials_set = set()
+        d = max_degree - min_degree
+        for item in it:
+            count = 0
+            for variable in item:
+                if variable == 1:
+                    count += 1
+                    if d < count:
+                        break
+            else:
+                monomials_set.add(Mul(*item))
+        yield from monomials_set
 
 def monomial_count(V, N):
     r"""
@@ -182,7 +169,7 @@ def monomial_count(V, N):
     6
 
     """
-    from sympy import factorial
+    from sympy.functions.combinatorial.factorials import factorial
     return factorial(V + N) / factorial(V) / factorial(N)
 
 def monomial_mul(A, B):
@@ -400,17 +387,24 @@ def term_div(a, b, domain):
 class MonomialOps:
     """Code generator of fast monomial arithmetic functions. """
 
-    def __init__(self, ngens):
-        self.ngens = ngens
+    @cacheit
+    def __new__(cls, ngens):
+        obj = super().__new__(cls)
+        obj.ngens = ngens
+        return obj
+
+    def __getnewargs__(self):
+        return (self.ngens,)
 
     def _build(self, code, name):
         ns = {}
-        exec_(code, ns)
+        exec(code, ns)
         return ns[name]
 
     def _vars(self, name):
         return [ "%s%s" % (name, i) for i in range(self.ngens) ]
 
+    @cacheit
     def mul(self):
         name = "monomial_mul"
         template = dedent("""\
@@ -422,9 +416,10 @@ class MonomialOps:
         A = self._vars("a")
         B = self._vars("b")
         AB = [ "%s + %s" % (a, b) for a, b in zip(A, B) ]
-        code = template % dict(name=name, A=", ".join(A), B=", ".join(B), AB=", ".join(AB))
+        code = template % {"name": name, "A": ", ".join(A), "B": ", ".join(B), "AB": ", ".join(AB)}
         return self._build(code, name)
 
+    @cacheit
     def pow(self):
         name = "monomial_pow"
         template = dedent("""\
@@ -434,9 +429,10 @@ class MonomialOps:
         """)
         A = self._vars("a")
         Ak = [ "%s*k" % a for a in A ]
-        code = template % dict(name=name, A=", ".join(A), Ak=", ".join(Ak))
+        code = template % {"name": name, "A": ", ".join(A), "Ak": ", ".join(Ak)}
         return self._build(code, name)
 
+    @cacheit
     def mulpow(self):
         name = "monomial_mulpow"
         template = dedent("""\
@@ -448,9 +444,10 @@ class MonomialOps:
         A = self._vars("a")
         B = self._vars("b")
         ABk = [ "%s + %s*k" % (a, b) for a, b in zip(A, B) ]
-        code = template % dict(name=name, A=", ".join(A), B=", ".join(B), ABk=", ".join(ABk))
+        code = template % {"name": name, "A": ", ".join(A), "B": ", ".join(B), "ABk": ", ".join(ABk)}
         return self._build(code, name)
 
+    @cacheit
     def ldiv(self):
         name = "monomial_ldiv"
         template = dedent("""\
@@ -462,9 +459,10 @@ class MonomialOps:
         A = self._vars("a")
         B = self._vars("b")
         AB = [ "%s - %s" % (a, b) for a, b in zip(A, B) ]
-        code = template % dict(name=name, A=", ".join(A), B=", ".join(B), AB=", ".join(AB))
+        code = template % {"name": name, "A": ", ".join(A), "B": ", ".join(B), "AB": ", ".join(AB)}
         return self._build(code, name)
 
+    @cacheit
     def div(self):
         name = "monomial_div"
         template = dedent("""\
@@ -476,11 +474,12 @@ class MonomialOps:
         """)
         A = self._vars("a")
         B = self._vars("b")
-        RAB = [ "r%(i)s = a%(i)s - b%(i)s\n    if r%(i)s < 0: return None" % dict(i=i) for i in range(self.ngens) ]
+        RAB = [ "r%(i)s = a%(i)s - b%(i)s\n    if r%(i)s < 0: return None" % {"i": i} for i in range(self.ngens) ]
         R = self._vars("r")
-        code = template % dict(name=name, A=", ".join(A), B=", ".join(B), RAB="\n    ".join(RAB), R=", ".join(R))
+        code = template % {"name": name, "A": ", ".join(A), "B": ", ".join(B), "RAB": "\n    ".join(RAB), "R": ", ".join(R)}
         return self._build(code, name)
 
+    @cacheit
     def lcm(self):
         name = "monomial_lcm"
         template = dedent("""\
@@ -492,9 +491,10 @@ class MonomialOps:
         A = self._vars("a")
         B = self._vars("b")
         AB = [ "%s if %s >= %s else %s" % (a, a, b, b) for a, b in zip(A, B) ]
-        code = template % dict(name=name, A=", ".join(A), B=", ".join(B), AB=", ".join(AB))
+        code = template % {"name": name, "A": ", ".join(A), "B": ", ".join(B), "AB": ", ".join(AB)}
         return self._build(code, name)
 
+    @cacheit
     def gcd(self):
         name = "monomial_gcd"
         template = dedent("""\
@@ -506,7 +506,7 @@ class MonomialOps:
         A = self._vars("a")
         B = self._vars("b")
         AB = [ "%s if %s <= %s else %s" % (a, a, b, b) for a, b in zip(A, B) ]
-        code = template % dict(name=name, A=", ".join(A), B=", ".join(B), AB=", ".join(AB))
+        code = template % {"name": name, "A": ", ".join(A), "B": ", ".join(B), "AB": ", ".join(AB)}
         return self._build(code, name)
 
 @public
@@ -553,7 +553,7 @@ class Monomial(PicklableWithSlots):
 
         if not gens:
             raise ValueError(
-                "can't convert %s to an expression without generators" % self)
+                "Cannot convert %s to an expression without generators" % self)
 
         return Mul(*[ gen**exp for gen, exp in zip(gens, self.exponents) ])
 
@@ -599,18 +599,9 @@ class Monomial(PicklableWithSlots):
 
     def __pow__(self, other):
         n = int(other)
-
-        if not n:
-            return self.rebuild([0]*len(self))
-        elif n > 0:
-            exponents = self.exponents
-
-            for i in range(1, n):
-                exponents = monomial_mul(exponents, self.exponents)
-
-            return self.rebuild(exponents)
-        else:
+        if n < 0:
             raise ValueError("a non-negative integer expected, got %s" % other)
+        return self.rebuild(monomial_pow(self.exponents, n))
 
     def gcd(self, other):
         """Greatest common divisor of monomials. """
