@@ -5,6 +5,7 @@ from operator import add, mul
 
 from sympy.polys.rings import ring, xring, sring, PolyRing, PolyElement
 from sympy.polys.fields import field, FracField
+from sympy.polys.densebasic import ninf
 from sympy.polys.domains import ZZ, QQ, RR, FF, EX
 from sympy.polys.orderings import lex, grlex
 from sympy.polys.polyerrors import GeneratorsError, \
@@ -12,8 +13,10 @@ from sympy.polys.polyerrors import GeneratorsError, \
 
 from sympy.testing.pytest import raises
 from sympy.core import Symbol, symbols
-
-from sympy import sqrt, pi, oo, exp
+from sympy.core.singleton import S
+from sympy.core.numbers import pi
+from sympy.functions.elementary.exponential import exp
+from sympy.functions.elementary.miscellaneous import sqrt
 
 def test_PolyRing___init__():
     x, y, z, t = map(Symbol, "xyzt")
@@ -55,19 +58,10 @@ def test_PolyRing___hash__():
 
 def test_PolyRing___eq__():
     assert ring("x,y,z", QQ)[0] == ring("x,y,z", QQ)[0]
-    assert ring("x,y,z", QQ)[0] is ring("x,y,z", QQ)[0]
-
     assert ring("x,y,z", QQ)[0] != ring("x,y,z", ZZ)[0]
-    assert ring("x,y,z", QQ)[0] is not ring("x,y,z", ZZ)[0]
-
     assert ring("x,y,z", ZZ)[0] != ring("x,y,z", QQ)[0]
-    assert ring("x,y,z", ZZ)[0] is not ring("x,y,z", QQ)[0]
-
     assert ring("x,y,z", QQ)[0] != ring("x,y", QQ)[0]
-    assert ring("x,y,z", QQ)[0] is not ring("x,y", QQ)[0]
-
     assert ring("x,y", QQ)[0] != ring("x,y,z", QQ)[0]
-    assert ring("x,y", QQ)[0] is not ring("x,y,z", QQ)[0]
 
 def test_PolyRing_ring_new():
     R, x, y, z = ring("x,y,z", QQ)
@@ -148,6 +142,18 @@ def test_PolyRing_mul():
 
     assert R.mul([2, 3, 5]) == 30
 
+def test_PolyRing_symmetric_poly():
+    R, x, y, z, t = ring("x,y,z,t", ZZ)
+
+    raises(ValueError, lambda: R.symmetric_poly(-1))
+    raises(ValueError, lambda: R.symmetric_poly(5))
+
+    assert R.symmetric_poly(0) == R.one
+    assert R.symmetric_poly(1) == x + y + z + t
+    assert R.symmetric_poly(2) == x*y + x*z + x*t + y*z + y*t + z*t
+    assert R.symmetric_poly(3) == x*y*z + x*y*t + x*z*t + y*z*t
+    assert R.symmetric_poly(4) == x*y*z*t
+
 def test_sring():
     x, y, z, t = symbols("x,y,z,t")
 
@@ -226,6 +232,17 @@ def test_PolyElement__lt_le_gt_ge__():
     assert x**3 > x**2 > x > R(1)
     assert x**3 >= x**2 >= x >= R(1)
 
+def test_PolyElement__str__():
+    x, y = symbols('x, y')
+
+    for dom in [ZZ, QQ, ZZ[x], ZZ[x,y], ZZ[x][y]]:
+        R, t = ring('t', dom)
+        assert str(2*t**2 + 1) == '2*t**2 + 1'
+
+    for dom in [EX, EX[x]]:
+        R, t = ring('t', dom)
+        assert str(2*t**2 + 1) == 'EX(2)*t**2 + EX(1)'
+
 def test_PolyElement_copy():
     R, x, y, z = ring("x,y,z", ZZ)
 
@@ -246,39 +263,39 @@ def test_PolyElement_as_expr():
     assert f != g
     assert f.as_expr() == g
 
-    X, Y, Z = symbols("x,y,z")
-    g = 3*X**2*Y - X*Y*Z + 7*Z**3 + 1
+    U, V, W = symbols("u,v,w")
+    g = 3*U**2*V - U*V*W + 7*W**3 + 1
 
     assert f != g
-    assert f.as_expr(X, Y, Z) == g
+    assert f.as_expr(U, V, W) == g
 
     raises(ValueError, lambda: f.as_expr(X))
 
     R, = ring("", ZZ)
-    R(3).as_expr() == 3
+    assert R(3).as_expr() == 3
 
 def test_PolyElement_from_expr():
     x, y, z = symbols("x,y,z")
     R, X, Y, Z = ring((x, y, z), ZZ)
 
     f = R.from_expr(1)
-    assert f == 1 and isinstance(f, R.dtype)
+    assert f == 1 and R.is_element(f)
 
     f = R.from_expr(x)
-    assert f == X and isinstance(f, R.dtype)
+    assert f == X and R.is_element(f)
 
     f = R.from_expr(x*y*z)
-    assert f == X*Y*Z and isinstance(f, R.dtype)
+    assert f == X*Y*Z and R.is_element(f)
 
     f = R.from_expr(x*y*z + x*y + x)
-    assert f == X*Y*Z + X*Y + X and isinstance(f, R.dtype)
+    assert f == X*Y*Z + X*Y + X and R.is_element(f)
 
     f = R.from_expr(x**3*y*z + x**2*y**7 + 1)
-    assert f == X**3*Y*Z + X**2*Y**7 + 1 and isinstance(f, R.dtype)
+    assert f == X**3*Y*Z + X**2*Y**7 + 1 and R.is_element(f)
 
     r, F = sring([exp(2)])
     f = r.from_expr(exp(2))
-    assert f == F[0] and isinstance(f, r.dtype)
+    assert f == F[0] and r.is_element(f)
 
     raises(ValueError, lambda: R.from_expr(1/x))
     raises(ValueError, lambda: R.from_expr(2**x))
@@ -286,33 +303,35 @@ def test_PolyElement_from_expr():
 
     R, = ring("", ZZ)
     f = R.from_expr(1)
-    assert f == 1 and isinstance(f, R.dtype)
+    assert f == 1 and R.is_element(f)
 
 def test_PolyElement_degree():
     R, x,y,z = ring("x,y,z", ZZ)
 
-    assert R(0).degree() is -oo
+    assert ninf == float('-inf')
+
+    assert R(0).degree() is ninf
     assert R(1).degree() == 0
     assert (x + 1).degree() == 1
     assert (2*y**3 + z).degree() == 0
     assert (x*y**3 + z).degree() == 1
     assert (x**5*y**3 + z).degree() == 5
 
-    assert R(0).degree(x) is -oo
+    assert R(0).degree(x) is ninf
     assert R(1).degree(x) == 0
     assert (x + 1).degree(x) == 1
     assert (2*y**3 + z).degree(x) == 0
     assert (x*y**3 + z).degree(x) == 1
     assert (7*x**5*y**3 + z).degree(x) == 5
 
-    assert R(0).degree(y) is -oo
+    assert R(0).degree(y) is ninf
     assert R(1).degree(y) == 0
     assert (x + 1).degree(y) == 0
     assert (2*y**3 + z).degree(y) == 3
     assert (x*y**3 + z).degree(y) == 3
     assert (7*x**5*y**3 + z).degree(y) == 3
 
-    assert R(0).degree(z) is -oo
+    assert R(0).degree(z) is ninf
     assert R(1).degree(z) == 0
     assert (x + 1).degree(z) == 0
     assert (2*y**3 + z).degree(z) == 1
@@ -320,34 +339,34 @@ def test_PolyElement_degree():
     assert (7*x**5*y**3 + z).degree(z) == 1
 
     R, = ring("", ZZ)
-    assert R(0).degree() is -oo
+    assert R(0).degree() is ninf
     assert R(1).degree() == 0
 
 def test_PolyElement_tail_degree():
     R, x,y,z = ring("x,y,z", ZZ)
 
-    assert R(0).tail_degree() is -oo
+    assert R(0).tail_degree() is ninf
     assert R(1).tail_degree() == 0
     assert (x + 1).tail_degree() == 0
     assert (2*y**3 + x**3*z).tail_degree() == 0
     assert (x*y**3 + x**3*z).tail_degree() == 1
     assert (x**5*y**3 + x**3*z).tail_degree() == 3
 
-    assert R(0).tail_degree(x) is -oo
+    assert R(0).tail_degree(x) is ninf
     assert R(1).tail_degree(x) == 0
     assert (x + 1).tail_degree(x) == 0
     assert (2*y**3 + x**3*z).tail_degree(x) == 0
     assert (x*y**3 + x**3*z).tail_degree(x) == 1
     assert (7*x**5*y**3 + x**3*z).tail_degree(x) == 3
 
-    assert R(0).tail_degree(y) is -oo
+    assert R(0).tail_degree(y) is ninf
     assert R(1).tail_degree(y) == 0
     assert (x + 1).tail_degree(y) == 0
     assert (2*y**3 + x**3*z).tail_degree(y) == 0
     assert (x*y**3 + x**3*z).tail_degree(y) == 0
     assert (7*x**5*y**3 + x**3*z).tail_degree(y) == 0
 
-    assert R(0).tail_degree(z) is -oo
+    assert R(0).tail_degree(z) is ninf
     assert R(1).tail_degree(z) == 0
     assert (x + 1).tail_degree(z) == 0
     assert (2*y**3 + x**3*z).tail_degree(z) == 0
@@ -355,20 +374,20 @@ def test_PolyElement_tail_degree():
     assert (7*x**5*y**3 + x**3*z).tail_degree(z) == 0
 
     R, = ring("", ZZ)
-    assert R(0).tail_degree() is -oo
+    assert R(0).tail_degree() is ninf
     assert R(1).tail_degree() == 0
 
 def test_PolyElement_degrees():
     R, x,y,z = ring("x,y,z", ZZ)
 
-    assert R(0).degrees() == (-oo, -oo, -oo)
+    assert R(0).degrees() == (ninf, ninf, ninf)
     assert R(1).degrees() == (0, 0, 0)
     assert (x**2*y + x**3*z**2).degrees() == (3, 1, 2)
 
 def test_PolyElement_tail_degrees():
     R, x,y,z = ring("x,y,z", ZZ)
 
-    assert R(0).tail_degrees() == (-oo, -oo, -oo)
+    assert R(0).tail_degrees() == (ninf, ninf, ninf)
     assert R(1).tail_degrees() == (0, 0, 0)
     assert (x**2*y + x**3*z**2).tail_degrees() == (2, 0, 0)
 
@@ -392,7 +411,7 @@ def test_PolyElement_coeff():
     raises(ValueError, lambda: f.coeff(7*z**3))
 
     R, = ring("", ZZ)
-    R(3).coeff(1) == 3
+    assert R(3).coeff(1) == 3
 
 def test_PolyElement_LC():
     R, x, y = ring("x,y", QQ, lex)
@@ -587,9 +606,9 @@ def test_PolyElement___truediv__():
     assert (x**2 - 1).quo(x) == x
     assert (x**2 - x).quo(x) == x - 1
 
-    assert (x**2 - 1)/x == x - x**(-1)
+    raises(ExactQuotientFailed, lambda: (x**2 - 1)/x)
     assert (x**2 - x)/x == x - 1
-    assert (x**2 - 1)/(2*x) == x/2 - x**(-1)/2
+    raises(ExactQuotientFailed, lambda: (x**2 - 1)/(2*x))
 
     assert (x**2 - 1).quo(2*x) == 0
     assert (x**2 - x)/(x - 1) == (x**2 - x).quo(x - 1) == x
@@ -606,7 +625,7 @@ def test_PolyElement___truediv__():
     Rxyz, x,y,z = ring("x,y,z", Ruv)
 
     assert dict((u**2*x + u)/u) == {(1, 0, 0): u, (0, 0, 0): 1}
-    raises(TypeError, lambda: u/(u**2*x + u))
+    raises(ExactQuotientFailed, lambda: u/(u**2*x + u))
 
     raises(TypeError, lambda: t/x)
     raises(TypeError, lambda: x/t)
@@ -642,7 +661,8 @@ def test_PolyElement___truediv__():
 
     assert f.div(g) == divmod(f, g) == (q, r)
     assert f.rem(g) == f % g == r
-    assert f.quo(g) == f / g == q
+    assert f.quo(g) == q
+    raises(ExactQuotientFailed, lambda: f / g)
     raises(ExactQuotientFailed, lambda: f.exquo(g))
 
     f, g = 3*x**3 + x**2 + x + 5, 5*x**2 - 3*x + 1
@@ -650,7 +670,8 @@ def test_PolyElement___truediv__():
 
     assert f.div(g) == divmod(f, g) == (q, r)
     assert f.rem(g) == f % g == r
-    assert f.quo(g) == f / g == q
+    assert f.quo(g) == q
+    raises(ExactQuotientFailed, lambda: f / g)
     raises(ExactQuotientFailed, lambda: f.exquo(g))
 
     f, g = 5*x**4 + 4*x**3 + 3*x**2 + 2*x + 1, x**2 + 2*x + 3
@@ -658,7 +679,8 @@ def test_PolyElement___truediv__():
 
     assert f.div(g) == divmod(f, g) == (q, r)
     assert f.rem(g) == f % g == r
-    assert f.quo(g) == f / g == q
+    assert f.quo(g) == q
+    raises(ExactQuotientFailed, lambda: f / g)
     raises(ExactQuotientFailed, lambda: f.exquo(g))
 
     f, g = 5*x**5 + 4*x**4 + 3*x**3 + 2*x**2 + x, x**4 + 2*x**3 + 9
@@ -666,7 +688,8 @@ def test_PolyElement___truediv__():
 
     assert f.div(g) == divmod(f, g) == (q, r)
     assert f.rem(g) == f % g == r
-    assert f.quo(g) == f / g == q
+    assert f.quo(g) == q
+    raises(ExactQuotientFailed, lambda: f / g)
     raises(ExactQuotientFailed, lambda: f.exquo(g))
 
     R, x = ring("x", QQ)
@@ -676,7 +699,8 @@ def test_PolyElement___truediv__():
 
     assert f.div(g) == divmod(f, g) == (q, r)
     assert f.rem(g) == f % g == r
-    assert f.quo(g) == f / g == q
+    assert f.quo(g) == q
+    raises(ExactQuotientFailed, lambda: f / g)
     raises(ExactQuotientFailed, lambda: f.exquo(g))
 
     f, g = 3*x**3 + x**2 + x + 5, 5*x**2 - 3*x + 1
@@ -684,7 +708,8 @@ def test_PolyElement___truediv__():
 
     assert f.div(g) == divmod(f, g) == (q, r)
     assert f.rem(g) == f % g == r
-    assert f.quo(g) == f / g == q
+    assert f.quo(g) == q
+    raises(ExactQuotientFailed, lambda: f / g)
     raises(ExactQuotientFailed, lambda: f.exquo(g))
 
     R, x,y = ring("x,y", ZZ)
@@ -694,15 +719,16 @@ def test_PolyElement___truediv__():
 
     assert f.div(g) == divmod(f, g) == (q, r)
     assert f.rem(g) == f % g == r
-    assert f.quo(g) == f / g == q
-    assert f.exquo(g) == q
+    assert f.quo(g) == q
+    assert f.exquo(g) == f / g == q
 
     f, g = x**2 + y**2, x - y
     q, r = x + y, 2*y**2
 
     assert f.div(g) == divmod(f, g) == (q, r)
     assert f.rem(g) == f % g == r
-    assert f.quo(g) == f / g == q
+    assert f.quo(g) == q
+    raises(ExactQuotientFailed, lambda: f / g)
     raises(ExactQuotientFailed, lambda: f.exquo(g))
 
     f, g = x**2 + y**2, -x + y
@@ -710,7 +736,8 @@ def test_PolyElement___truediv__():
 
     assert f.div(g) == divmod(f, g) == (q, r)
     assert f.rem(g) == f % g == r
-    assert f.quo(g) == f / g == q
+    assert f.quo(g) == q
+    raises(ExactQuotientFailed, lambda: f / g)
     raises(ExactQuotientFailed, lambda: f.exquo(g))
 
     f, g = x**2 + y**2, 2*x - 2*y
@@ -718,7 +745,8 @@ def test_PolyElement___truediv__():
 
     assert f.div(g) == divmod(f, g) == (q, r)
     assert f.rem(g) == f % g == r
-    assert f.quo(g) == f / g == q
+    assert f.quo(g) == q
+    raises(ExactQuotientFailed, lambda: f / g)
     raises(ExactQuotientFailed, lambda: f.exquo(g))
 
     R, x,y = ring("x,y", QQ)
@@ -728,15 +756,16 @@ def test_PolyElement___truediv__():
 
     assert f.div(g) == divmod(f, g) == (q, r)
     assert f.rem(g) == f % g == r
-    assert f.quo(g) == f / g == q
-    assert f.exquo(g) == q
+    assert f.quo(g) == q
+    assert f.exquo(g) == f / g == q
 
     f, g = x**2 + y**2, x - y
     q, r = x + y, 2*y**2
 
     assert f.div(g) == divmod(f, g) == (q, r)
     assert f.rem(g) == f % g == r
-    assert f.quo(g) == f / g == q
+    assert f.quo(g) == q
+    raises(ExactQuotientFailed, lambda: f / g)
     raises(ExactQuotientFailed, lambda: f.exquo(g))
 
     f, g = x**2 + y**2, -x + y
@@ -744,7 +773,8 @@ def test_PolyElement___truediv__():
 
     assert f.div(g) == divmod(f, g) == (q, r)
     assert f.rem(g) == f % g == r
-    assert f.quo(g) == f / g == q
+    assert f.quo(g) == q
+    raises(ExactQuotientFailed, lambda: f / g)
     raises(ExactQuotientFailed, lambda: f.exquo(g))
 
     f, g = x**2 + y**2, 2*x - 2*y
@@ -752,7 +782,8 @@ def test_PolyElement___truediv__():
 
     assert f.div(g) == divmod(f, g) == (q, r)
     assert f.rem(g) == f % g == r
-    assert f.quo(g) == f / g == q
+    assert f.quo(g) == q
+    raises(ExactQuotientFailed, lambda: f / g)
     raises(ExactQuotientFailed, lambda: f.exquo(g))
 
 def test_PolyElement___pow__():
@@ -763,8 +794,6 @@ def test_PolyElement___pow__():
     assert f**1 == f
     raises(ValueError, lambda: f**(-1))
 
-    assert x**(-1) == x**(-1)
-
     assert f**2 == f._pow_generic(2) == f._pow_multinomial(2) == 4*x**2 + 12*x + 9
     assert f**3 == f._pow_generic(3) == f._pow_multinomial(3) == 8*x**3 + 36*x**2 + 54*x + 27
     assert f**4 == f._pow_generic(4) == f._pow_multinomial(4) == 16*x**4 + 96*x**3 + 216*x**2 + 216*x + 81
@@ -1144,13 +1173,13 @@ def test_PolyElement_evaluate():
     f = (x*y)**3 + 4*(x*y)**2 + 2*x*y + 3
 
     r = f.evaluate(x, 0)
-    assert r == 3 and isinstance(r, R.drop(x).dtype)
+    assert r == 3 and R.drop(x).is_element(r)
     r = f.evaluate([(x, 0), (y, 0)])
-    assert r == 3 and isinstance(r, R.drop(x, y).dtype)
+    assert r == 3 and R.drop(x, y).is_element(r)
     r = f.evaluate(y, 0)
-    assert r == 3 and isinstance(r, R.drop(y).dtype)
+    assert r == 3 and R.drop(y).is_element(r)
     r = f.evaluate([(y, 0), (x, 0)])
-    assert r == 3 and isinstance(r, R.drop(y, x).dtype)
+    assert r == 3 and R.drop(y, x).is_element(r)
 
     r = f.evaluate([(x, 0), (y, 0), (z, 0)])
     assert r == 3 and not isinstance(r, PolyElement)
@@ -1164,7 +1193,7 @@ def test_PolyElement_subs():
     f = x**3 + 4*x**2 + 2*x + 3
 
     r = f.subs(x, 0)
-    assert r == 3 and isinstance(r, R.dtype)
+    assert r == 3 and R.is_element(r)
 
     raises(CoercionFailed, lambda: f.subs(x, QQ(1,7)))
 
@@ -1172,20 +1201,59 @@ def test_PolyElement_subs():
     f = x**3 + 4*x**2 + 2*x + 3
 
     r = f.subs(x, 0)
-    assert r == 3 and isinstance(r, R.dtype)
+    assert r == 3 and R.is_element(r)
     r = f.subs([(x, 0), (y, 0)])
-    assert r == 3 and isinstance(r, R.dtype)
+    assert r == 3 and R.is_element(r)
 
     raises(CoercionFailed, lambda: f.subs([(x, 1), (y, QQ(1,7))]))
     raises(CoercionFailed, lambda: f.subs([(x, QQ(1,7)), (y, 1)]))
     raises(CoercionFailed, lambda: f.subs([(x, QQ(1,7)), (y, QQ(1,7))]))
 
+def test_PolyElement_symmetrize():
+    R, x, y = ring("x,y", ZZ)
+
+    # Homogeneous, symmetric
+    f = x**2 + y**2
+    sym, rem, m = f.symmetrize()
+    assert rem == 0
+    assert sym.compose(m) + rem == f
+
+    # Homogeneous, asymmetric
+    f = x**2 - y**2
+    sym, rem, m = f.symmetrize()
+    assert rem != 0
+    assert sym.compose(m) + rem == f
+
+    # Inhomogeneous, symmetric
+    f = x*y + 7
+    sym, rem, m = f.symmetrize()
+    assert rem == 0
+    assert sym.compose(m) + rem == f
+
+    # Inhomogeneous, asymmetric
+    f = y + 7
+    sym, rem, m = f.symmetrize()
+    assert rem != 0
+    assert sym.compose(m) + rem == f
+
+    # Constant
+    f = R.from_expr(3)
+    sym, rem, m = f.symmetrize()
+    assert rem == 0
+    assert sym.compose(m) + rem == f
+
+    # Constant constructed from sring
+    R, f = sring(3)
+    sym, rem, m = f.symmetrize()
+    assert rem == 0
+    assert sym.compose(m) + rem == f
+
 def test_PolyElement_compose():
     R, x = ring("x", ZZ)
     f = x**3 + 4*x**2 + 2*x + 3
 
     r = f.compose(x, 0)
-    assert r == 3 and isinstance(r, R.dtype)
+    assert r == 3 and R.is_element(r)
 
     assert f.compose(x, x) == f
     assert f.compose(x, x**2) == x**6 + 4*x**4 + 2*x**2 + 3
@@ -1196,13 +1264,13 @@ def test_PolyElement_compose():
     f = x**3 + 4*x**2 + 2*x + 3
 
     r = f.compose(x, 0)
-    assert r == 3 and isinstance(r, R.dtype)
+    assert r == 3 and R.is_element(r)
     r = f.compose([(x, 0), (y, 0)])
-    assert r == 3 and isinstance(r, R.dtype)
+    assert r == 3 and R.is_element(r)
 
     r = (x**3 + 4*x**2 + 2*x*y*z + 3).compose(x, y*z**2 - 1)
     q = (y*z**2 - 1)**3 + 4*(y*z**2 - 1)**2 + 2*(y*z**2 - 1)*y*z + 3
-    assert r == q and isinstance(r, R.dtype)
+    assert r == q and R.is_element(r)
 
 def test_PolyElement_is_():
     R, x,y,z = ring("x,y,z", QQ)
@@ -1283,21 +1351,77 @@ def test_PolyElement_drop():
 
     assert R(1).drop(0).ring == PolyRing("y,z", ZZ, lex)
     assert R(1).drop(0).drop(0).ring == PolyRing("z", ZZ, lex)
-    assert isinstance(R(1).drop(0).drop(0).drop(0), R.dtype) is False
+    assert R.is_element(R(1).drop(0).drop(0).drop(0)) is False
 
     raises(ValueError, lambda: z.drop(0).drop(0).drop(0))
     raises(ValueError, lambda: x.drop(0))
 
+def test_PolyElement_coeff_wrt():
+    R, x, y, z = ring("x, y, z", ZZ)
+
+    p = 4*x**3 + 5*y**2 + 6*y**2*z + 7
+    assert p.coeff_wrt(1, 2) == 6*z + 5 # using generator index
+    assert p.coeff_wrt(x, 3) == 4 # using generator
+
+    p = 2*x**4 + 3*x*y**2*z + 10*y**2 + 10*x*z**2
+    assert p.coeff_wrt(x, 1) == 3*y**2*z + 10*z**2
+    assert p.coeff_wrt(y, 2) == 3*x*z + 10
+
+    p = 4*x**2 + 2*x*y + 5
+    assert p.coeff_wrt(z, 1) == R(0)
+    assert p.coeff_wrt(y, 2) == R(0)
+
+def test_PolyElement_prem():
+    R, x, y = ring("x, y", ZZ)
+
+    f, g = x**2 + x*y, 2*x + 2
+    assert f.prem(g) == -4*y + 4 # first generator is chosen by default if it is not given
+
+    f, g = x**2 + 1, 2*x - 4
+    assert f.prem(g) == f.prem(g, x) == 20
+    assert f.prem(g, 1) == R(0)
+
+    f, g = x*y + 2*x + 1, x + y
+    assert f.prem(g) == -y**2 - 2*y + 1
+    assert f.prem(g, 1) == f.prem(g, y) == -x**2 + 2*x + 1
+
+    raises(ZeroDivisionError, lambda: f.prem(R(0)))
+
 def test_PolyElement_pdiv():
-    _, x, y = ring("x,y", ZZ)
+    R, x, y = ring("x,y", ZZ)
+
+    f, g = x**4 + 5*x**3 + 7*x**2, 2*x**2 + 3
+    assert f.pdiv(g) == f.pdiv(g, x) == (4*x**2 + 20*x + 22, -60*x - 66)
 
     f, g = x**2 - y**2, x - y
-    q, r = x + y, 0
+    assert f.pdiv(g) == f.pdiv(g, 0) == (x + y, 0)
 
-    assert f.pdiv(g) == (q, r)
-    assert f.prem(g) == r
-    assert f.pquo(g) == q
-    assert f.pexquo(g) == q
+    f, g = x*y + 2*x + 1, x + y
+    assert f.pdiv(g) == (y + 2, -y**2 - 2*y + 1)
+    assert f.pdiv(g, y) == f.pdiv(g, 1) == (x + 1, -x**2 + 2*x + 1)
+
+    assert R(0).pdiv(g) == (0, 0)
+    raises(ZeroDivisionError, lambda: f.prem(R(0)))
+
+def test_PolyElement_pquo():
+    R, x, y = ring("x, y", ZZ)
+
+    f, g = x**4 - 4*x**2*y + 4*y**2, x**2 - 2*y
+    assert f.pquo(g) == f.pquo(g, x) == x**2 - 2*y
+    assert f.pquo(g, y) == 4*x**2 - 8*y + 4
+
+    f, g = x**4 - y**4, x**2 - y**2
+    assert f.pquo(g) == f.pquo(g, 0) == x**2 + y**2
+
+def test_PolyElement_pexquo():
+    R, x, y = ring("x, y", ZZ)
+
+    f, g = x**2 - y**2, x - y
+    assert f.pexquo(g) == f.pexquo(g, x) == x + y
+    assert f.pexquo(g, y) == f.pexquo(g, 1) == x + y + 1
+
+    f, g = x**2 + 3*x + 6, x + 2
+    raises(ExactQuotientFailed, lambda: f.pexquo(g))
 
 def test_PolyElement_gcdex():
     _, x = ring("x", QQ)
@@ -1309,10 +1433,31 @@ def test_PolyElement_gcdex():
     assert f.gcdex(g) == (s, t, h)
 
 def test_PolyElement_subresultants():
-    _, x = ring("x", ZZ)
-    f, g, h = x**2 - 2*x + 1, x**2 - 1, 2*x - 2
+    R, x, y = ring("x, y", ZZ)
 
-    assert f.subresultants(g) == [f, g, h]
+    f, g = x**2*y + x*y, x + y # degree(f, x) > degree(g, x)
+    h = y**3 - y**2
+    assert f.subresultants(g) == [f, g, h] # first generator is chosen default
+
+    # generator index or generator is given
+    assert f.subresultants(g, 0) ==  f.subresultants(g, x) == [f, g, h]
+
+    assert f.subresultants(g, y) == [x**2*y + x*y, x + y, x**3 + x**2]
+
+    f, g = 2*x - y, x**2 + 2*y + x # degree(f, x) < degree(g, x)
+    assert f.subresultants(g) == [x**2 + x + 2*y, 2*x - y, y**2 + 10*y]
+
+    f, g = R(0), y**3 - y**2 # f = 0
+    assert f.subresultants(g) == [y**3 - y**2, 1]
+
+    f, g = x**2*y + x*y, R(0) # g = 0
+    assert f.subresultants(g) == [x**2*y + x*y, 1]
+
+    f, g = R(0), R(0) # f = 0 and g = 0
+    assert f.subresultants(g) == [0, 0]
+
+    f, g = x**2 + x, x**2 + x # f and g are same polynomial
+    assert f.subresultants(g) == [x**2 + x, x**2 + x]
 
 def test_PolyElement_resultant():
     _, x = ring("x", ZZ)
@@ -1346,6 +1491,12 @@ def test_PolyElement_decompose():
 def test_PolyElement_shift():
     _, x = ring("x", ZZ)
     assert (x**2 - 2*x + 1).shift(2) == x**2 + 2*x + 1
+    assert (x**2 - 2*x + 1).shift_list([2]) == x**2 + 2*x + 1
+
+    R, x, y = ring("x, y", ZZ)
+    assert (x*y).shift_list([1, 2]) == (x+1)*(y+2)
+
+    raises(MultivariatePolynomialError, lambda: (x*y).shift(1))
 
 def test_PolyElement_sturm():
     F, t = field("t", ZZ)
@@ -1369,16 +1520,25 @@ def test_PolyElement_gff_list():
     f = x*(x - 1)**3*(x - 2)**2*(x - 4)**2*(x - 5)
     assert f.gff_list() == [(x**2 - 5*x + 4, 1), (x**2 - 5*x + 4, 2), (x, 3)]
 
+def test_PolyElement_norm():
+    k = QQ
+    K = QQ.algebraic_field(sqrt(2))
+    sqrt2 = K.unit
+    _, X, Y = ring("x,y", k)
+    _, x, y = ring("x,y", K)
+
+    assert (x*y + sqrt2).norm() == X**2*Y**2 - 2
+
 def test_PolyElement_sqf_norm():
     R, x = ring("x", QQ.algebraic_field(sqrt(3)))
     X = R.to_ground().x
 
-    assert (x**2 - 2).sqf_norm() == (1, x**2 - 2*sqrt(3)*x + 1, X**4 - 10*X**2 + 1)
+    assert (x**2 - 2).sqf_norm() == ([1], x**2 - 2*sqrt(3)*x + 1, X**4 - 10*X**2 + 1)
 
     R, x = ring("x", QQ.algebraic_field(sqrt(2)))
     X = R.to_ground().x
 
-    assert (x**2 - 3).sqf_norm() == (1, x**2 - 2*sqrt(2)*x - 1, X**4 - 10*X**2 + 1)
+    assert (x**2 - 3).sqf_norm() == ([1], x**2 - 2*sqrt(2)*x - 1, X**4 - 10*X**2 + 1)
 
 def test_PolyElement_sqf_list():
     _, x = ring("x", ZZ)
@@ -1391,6 +1551,16 @@ def test_PolyElement_sqf_list():
     assert f.sqf_part() == p
     assert f.sqf_list() == (1, [(g, 1), (h, 2)])
 
+def test_issue_18894():
+    items = [S(3)/16 + sqrt(3*sqrt(3) + 10)/8, S(1)/8 + 3*sqrt(3)/16, S(1)/8 + 3*sqrt(3)/16, -S(3)/16 + sqrt(3*sqrt(3) + 10)/8]
+    R, a = sring(items, extension=True)
+    assert R.domain == QQ.algebraic_field(sqrt(3)+sqrt(3*sqrt(3)+10))
+    assert R.gens == ()
+    result = []
+    for item in items:
+        result.append(R.domain.from_sympy(item))
+    assert a == result
+
 def test_PolyElement_factor_list():
     _, x = ring("x", ZZ)
 
@@ -1407,3 +1577,15 @@ def test_issue_21410():
     R, x = ring('x', FF(2))
     p = x**6 + x**5 + x**4 + x**3 + 1
     assert p._pow_multinomial(4) == x**24 + x**20 + x**16 + x**12 + 1
+
+
+def test_zero_polynomial_primitive():
+
+    x = symbols('x')
+
+    R = ZZ[x]
+    zero_poly = R(0)
+    cont, prim = zero_poly.primitive()
+    assert cont == 0
+    assert prim == zero_poly
+    assert prim.is_primitive is False
