@@ -142,13 +142,13 @@ def test_constants_other():
 
 
 def test_boolean():
-    assert maple_code(x & y) == "x && y"
-    assert maple_code(x | y) == "x || y"
-    assert maple_code(~x) == "!x"
-    assert maple_code(x & y & z) == "x && y && z"
-    assert maple_code(x | y | z) == "x || y || z"
-    assert maple_code((x & y) | z) == "z || x && y"
-    assert maple_code((x | y) & z) == "z && (x || y)"
+    assert maple_code(x & y) == "x and y"
+    assert maple_code(x | y) == "x or y"
+    assert maple_code(~x) == "not x"
+    assert maple_code(x & y & z) == "x and y and z"
+    assert maple_code(x | y | z) == "x or y or z"
+    assert maple_code((x & y) | z) == "z or x and y"
+    assert maple_code((x | y) & z) == "z and (x or y)"
 
 
 def test_Matrices():
@@ -303,12 +303,8 @@ def test_sparse():
 
 # Not an important point.
 def test_maple_not_supported():
-    assert maple_code(S.ComplexInfinity) == (
-        "# Not supported in maple:\n"
-        "# ComplexInfinity\n"
-        "zoo"
-    )  # PROBLEM
-
+    with raises(NotImplementedError):
+        maple_code(S.ComplexInfinity)
 
 
 def test_MatrixElement_printing():
@@ -376,8 +372,8 @@ def test_maple_derivatives():
 
 
 def test_automatic_rewrites():
-    assert maple_code(lucas(x)) == '2^(-x)*((1 - sqrt(5))^x + (1 + sqrt(5))^x)'
-    assert maple_code(sinc(x)) == 'piecewise(x <> 0, sin(x)/x, 1)'
+    assert maple_code(lucas(x)) == '(2^(-x)*((1 - sqrt(5))^x + (1 + sqrt(5))^x))'
+    assert maple_code(sinc(x)) == '(piecewise(x <> 0, sin(x)/x, 1))'
 
 
 def test_specfun():
