@@ -118,8 +118,9 @@ debug this function to figure out the exact problem.
 """
 from functools import reduce
 
-from sympy.core import Basic, S, Mul, PoleError, expand_mul
+from sympy.core import Basic, S, Mul, PoleError
 from sympy.core.cache import cacheit
+from sympy.core.function import AppliedUndef
 from sympy.core.intfunc import ilcm
 from sympy.core.numbers import I, oo
 from sympy.core.symbol import Dummy, Wild
@@ -127,7 +128,6 @@ from sympy.core.traversal import bottom_up
 
 from sympy.functions import log, exp, sign as _sign
 from sympy.series.order import Order
-from sympy.utilities.exceptions import SymPyDeprecationWarning
 from sympy.utilities.misc import debug_decorator as debug
 from sympy.utilities.timeutils import timethis
 
@@ -275,9 +275,6 @@ def mrv(e, x):
         if b1 == 1:
             return SubsSet(), b1
         if e1.has(x):
-            if limitinf(b1, x) is S.One:
-                if limitinf(e1, x).is_infinite is False:
-                    return mrv(exp(e1*(b1 - 1)), x)
             return mrv(exp(e1*log(b1)), x)
         else:
             s, expr = mrv(b1, x)
@@ -304,6 +301,8 @@ def mrv(e, x):
         else:
             s, expr = mrv(e.exp, x)
             return s, exp(expr)
+    elif isinstance(e, AppliedUndef):
+        raise ValueError("MRV set computation for UndefinedFunction is not allowed")
     elif e.is_Function:
         l = [mrv(a, x) for a in e.args]
         l2 = [s for (s, _) in l if s != SubsSet()]
@@ -414,7 +413,7 @@ def sign(e, x):
             return 1
         if e.exp.is_Integer:
             return s**e.exp
-    elif isinstance(e, log):
+    elif isinstance(e, log) and e.args[0].is_positive:
         return sign(e.args[0] - 1, x)
 
     # if all else fails, do it the hard way
@@ -483,47 +482,6 @@ def moveup(l, x):
     return [e.xreplace({x: exp(x)}) for e in l]
 
 
-@debug
-@timeit
-def calculate_series(e, x, logx=None):
-    """ Calculates at least one term of the series of ``e`` in ``x``.
-
-    This is a place that fails most often, so it is in its own function.
-    """
-
-    SymPyDeprecationWarning(
-        feature="calculate_series",
-        useinstead="series() with suitable n, or as_leading_term",
-        issue=21838,
-        deprecated_since_version="1.12"
-    ).warn()
-
-    from sympy.simplify.powsimp import powdenest
-
-    for t in e.lseries(x, logx=logx):
-        # bottom_up function is required for a specific case - when e is
-        # -exp(p/(p + 1)) + exp(-p**2/(p + 1) + p)
-        t = bottom_up(t, lambda w:
-            getattr(w, 'normal', lambda: w)())
-        # And the expression
-        # `(-sin(1/x) + sin((x + exp(x))*exp(-x)/x))*exp(x)`
-        # from the first test of test_gruntz_eval_special needs to
-        # be expanded. But other forms need to be have at least
-        # factor_terms applied. `factor` accomplishes both and is
-        # faster than using `factor_terms` for the gruntz suite. It
-        # does not appear that use of `cancel` is necessary.
-        # t = cancel(t, expand=False)
-        t = t.factor()
-
-        if t.has(exp) and t.has(log):
-            t = powdenest(t)
-
-        if not t.is_zero:
-            break
-
-    return t
-
-
 @debug
 @timeit
 @cacheit
@@ -553,6 +511,11 @@ def mrv_leadterm(e, x):
     #
     w = Dummy("w", positive=True)
     f, logw = rewrite(exps, Omega, x, w)
+
+    # Ensure expressions of the form exp(log(...)) don't get simplified automatically in the previous steps.
+    # see: https://github.com/sympy/sympy/issues/15323#issuecomment-478639399
+    f = f.replace(lambda f: f.is_Pow and f.has(x), lambda f: exp(log(f.base)*f.exp))
+
     try:
         lt = f.leadterm(w, logx=logw)
     except (NotImplementedError, PoleError, ValueError):
@@ -660,7 +623,7 @@ def rewrite(e, Omega, x, wsym):
             if not isinstance(rewrites[var], exp):
                 raise ValueError("Value should be exp")
             arg = rewrites[var].args[0]
-        O2.append((var, exp((arg - c*g.exp).expand())*wsym**c))
+        O2.append((var, exp((arg - c*g.exp))*wsym**c))
 
     # Remember that Omega contains subexpressions of "e". So now we find
     # them in "e" and substitute them for our rewriting, stored in O2
@@ -691,7 +654,6 @@ def rewrite(e, Omega, x, wsym):
     # -exp(p/(p + 1)) + exp(-p**2/(p + 1) + p). No current simplification
     # methods reduce this to 0 while not expanding polynomials.
     f = bottom_up(f, lambda w: getattr(w, 'normal', lambda: w)())
-    f = expand_mul(f)
 
     return f, logw
 
