@@ -395,7 +395,7 @@ def test_fps__logarithmic_singularity():
 
 @XFAIL
 def test_fps__logarithmic_singularity_fail():
-    f = asech(x)  # Algorithms for computing limits probably needs improvemnts
+    f = asech(x)  # Algorithms for computing limits probably needs improvements
     assert fps(f, x) == log(2) - log(x) - x**2/4 - 3*x**4/64 + O(x**6)
 
 
@@ -416,12 +416,12 @@ def test_fps_symbolic():
 
     f = x**(n - 2)*cos(x)
     assert fps(f, x).truncate() == \
-        (x**(n - 2) - x**n/2 + x**(n + 2)/24 - x**(n + 4)/720 + O(x**(n + 6), x))
+        (x**(n - 2) - x**n/2 + x**(n + 2)/24 + O(x**(n + 4), x))
 
     f = x**(n - 2)*sin(x) + x**n*exp(x)
     assert fps(f, x).truncate() == \
-        (x**(n - 1) + x**n + 5*x**(n + 1)/6 + x**(n + 2)/2 + 7*x**(n + 3)/40 +
-         x**(n + 4)/24 + 41*x**(n + 5)/5040 + O(x**(n + 6), x))
+        (x**(n - 1) + x**(n + 1) + x**(n + 2)/2 + x**n +
+         x**(n + 4)/24 + x**(n + 5)/60 + O(x**(n + 6), x))
 
     f = x**n*atan(x)
     assert fps(f, x, oo).truncate() == \
