@@ -1,7 +1,17 @@
-from sympy import (
-    sqrt, Derivative, symbols, collect, Function, factor, Wild, S,
-    collect_const, log, fraction, I, cos, Add, O,sin, rcollect,
-    Mul, Pow, radsimp, diff, root, Symbol, Rational, exp, Abs)
+from sympy.core.add import Add
+from sympy.core.function import (Derivative, Function, diff)
+from sympy.core.mul import Mul
+from sympy.core.numbers import (I, Rational)
+from sympy.core.power import Pow
+from sympy.core.singleton import S
+from sympy.core.symbol import (Symbol, Wild, symbols)
+from sympy.functions.elementary.complexes import Abs
+from sympy.functions.elementary.exponential import (exp, log)
+from sympy.functions.elementary.miscellaneous import (root, sqrt)
+from sympy.functions.elementary.trigonometric import (cos, sin)
+from sympy.polys.polytools import factor
+from sympy.series.order import O
+from sympy.simplify.radsimp import (collect, collect_const, fraction, radsimp, rcollect)
 
 from sympy.core.expr import unchanged
 from sympy.core.mul import _unevaluated_Mul as umul
@@ -149,6 +159,14 @@ def test_radsimp():
     eq = sqrt(x)/y**2
     assert radsimp(eq) == eq
 
+    # handle non-Expr args
+    from sympy.integrals.integrals import Integral
+    eq = Integral(x/(sqrt(2) - 1), (x, 0, 1/(sqrt(2) + 1)))
+    assert radsimp(eq) == Integral((sqrt(2) + 1)*x , (x, 0, sqrt(2) - 1))
+
+    from sympy.sets import FiniteSet
+    eq = FiniteSet(x/(sqrt(2) - 1))
+    assert radsimp(eq) == FiniteSet((sqrt(2) + 1)*x)
 
 def test_radsimp_issue_3214():
     c, p = symbols('c p', positive=True)
@@ -158,7 +176,7 @@ def test_radsimp_issue_3214():
 
 
 def test_collect_1():
-    """Collect with respect to a Symbol"""
+    """Collect with respect to Symbol"""
     x, y, z, n = symbols('x,y,z,n')
     assert collect(1, x) == 1
     assert collect( x + y*x, x ) == x * (1 + y)
@@ -174,6 +192,11 @@ def test_collect_1():
     # symbols can be given as any iterable
     expr = x + y
     assert collect(expr, expr.free_symbols) == expr
+    assert collect(x*exp(x) + sin(x)*y + sin(x)*2 + 3*x, x, exact=None
+        ) == x*exp(x) + 3*x + (y + 2)*sin(x)
+    assert collect(x*exp(x) + sin(x)*y + sin(x)*2 + 3*x + y*x +
+        y*x*exp(x), x, exact=None
+        ) == x*exp(x)*(y + 1) + (3 + y)*x + (y + 2)*sin(x)
 
 
 def test_collect_2():
@@ -429,7 +452,8 @@ def test_issue_5615():
 
 
 def test_issue_5933():
-    from sympy import Polygon, RegularPolygon, denom
+    from sympy.geometry.polygon import (Polygon, RegularPolygon)
+    from sympy.simplify.radsimp import denom
     x = Polygon(*RegularPolygon((0, 0), 1, 5).vertices).centroid.x
     assert abs(denom(x).n()) > 1e-12
     assert abs(denom(radsimp(x))) > 1e-12  # in case simplify didn't handle it
@@ -467,3 +491,8 @@ def test_issue_19719():
     collected = collect(expr, (a**2, 1/a), evaluate=False)
     # Would return {_Dummy_20**(-2): b + 1, 1/a: 7 + 1/b} without xreplace
     assert collected == {a**2: b + 1, 1/a: 7 + 1/b}
+
+
+def test_issue_21355():
+    assert radsimp(1/(x + sqrt(x**2))) == 1/(x + sqrt(x**2))
+    assert radsimp(1/(x - sqrt(x**2))) == 1/(x - sqrt(x**2))
