@@ -2,6 +2,7 @@
 
 Contains
 ========
+FlorySchulz
 Geometric
 Hermite
 Logarithmic
@@ -14,14 +15,29 @@ Zeta
 
 
 
-from sympy import (Basic, factorial, exp, S, sympify, I, zeta, polylog, log, beta,
-                   hyper, binomial, Piecewise, floor, besseli, sqrt, Sum, Dummy,
-                   Lambda, Eq)
+from sympy.concrete.summations import Sum
+from sympy.core.basic import Basic
+from sympy.core.function import Lambda
+from sympy.core.numbers import I
+from sympy.core.relational import Eq
+from sympy.core.singleton import S
+from sympy.core.symbol import Dummy
+from sympy.core.sympify import sympify
+from sympy.functions.combinatorial.factorials import (binomial, factorial, FallingFactorial)
+from sympy.functions.elementary.exponential import (exp, log)
+from sympy.functions.elementary.integers import floor
+from sympy.functions.elementary.miscellaneous import sqrt
+from sympy.functions.elementary.piecewise import Piecewise
+from sympy.functions.special.bessel import besseli
+from sympy.functions.special.beta_functions import beta
+from sympy.functions.special.hyper import hyper
+from sympy.functions.special.zeta_functions import (polylog, zeta)
 from sympy.stats.drv import SingleDiscreteDistribution, SingleDiscretePSpace
 from sympy.stats.rv import _value_check, is_random
 
 
-__all__ = ['Geometric',
+__all__ = ['FlorySchulz',
+'Geometric',
 'Hermite',
 'Logarithmic',
 'NegativeBinomial',
@@ -60,6 +76,8 @@ class DiscreteDistributionHandmade(SingleDiscreteDistribution):
         val = Sum(pdf(x), (x, set._inf, set._sup)).doit()
         _value_check(Eq(val, 1) != S.false, "The pdf is incorrect on the given set.")
 
+
+
 def DiscreteRV(symbol, density, set=S.Integers, **kwargs):
     """
     Create a Discrete Random Variable given the following:
@@ -106,6 +124,77 @@ def DiscreteRV(symbol, density, set=S.Integers, **kwargs):
     return rv(symbol.name, DiscreteDistributionHandmade, pdf, set, **kwargs)
 
 
+#-------------------------------------------------------------------------------
+# Flory-Schulz distribution ------------------------------------------------------------
+
+class FlorySchulzDistribution(SingleDiscreteDistribution):
+    _argnames = ('a',)
+    set = S.Naturals
+
+    @staticmethod
+    def check(a):
+        _value_check((0 < a, a < 1), "a must be between 0 and 1")
+
+    def pdf(self, k):
+        a = self.a
+        return (a**2 * k * (1 - a)**(k - 1))
+
+    def _characteristic_function(self, t):
+        a = self.a
+        return a**2*exp(I*t)/((1 + (a - 1)*exp(I*t))**2)
+
+    def _moment_generating_function(self, t):
+        a = self.a
+        return a**2*exp(t)/((1 + (a - 1)*exp(t))**2)
+
+
+def FlorySchulz(name, a):
+    r"""
+    Create a discrete random variable with a FlorySchulz distribution.
+
+    The density of the FlorySchulz distribution is given by
+
+    .. math::
+        f(k) := (a^2) k (1 - a)^{k-1}
+
+    Parameters
+    ==========
+
+    a : A real number between 0 and 1
+
+    Returns
+    =======
+
+    RandomSymbol
+
+    Examples
+    ========
+
+    >>> from sympy.stats import density, E, variance, FlorySchulz
+    >>> from sympy import Symbol, S
+
+    >>> a = S.One / 5
+    >>> z = Symbol("z")
+
+    >>> X = FlorySchulz("x", a)
+
+    >>> density(X)(z)
+    (4/5)**(z - 1)*z/25
+
+    >>> E(X)
+    9
+
+    >>> variance(X)
+    40
+
+    References
+    ==========
+
+    https://en.wikipedia.org/wiki/Flory%E2%80%93Schulz_distribution
+    """
+    return rv(name, FlorySchulzDistribution, a)
+
+
 #-------------------------------------------------------------------------------
 # Geometric distribution ------------------------------------------------------------
 
@@ -133,6 +222,9 @@ def Geometric(name, p):
     r"""
     Create a discrete random variable with a Geometric distribution.
 
+    Explanation
+    ===========
+
     The density of the Geometric distribution is given by
 
     .. math::
@@ -141,7 +233,7 @@ def Geometric(name, p):
     Parameters
     ==========
 
-    p: A probability between 0 and 1
+    p : A probability between 0 and 1
 
     Returns
     =======
@@ -172,7 +264,7 @@ def Geometric(name, p):
     ==========
 
     .. [1] https://en.wikipedia.org/wiki/Geometric_distribution
-    .. [2] http://mathworld.wolfram.com/GeometricDistribution.html
+    .. [2] https://mathworld.wolfram.com/GeometricDistribution.html
 
     """
     return rv(name, GeometricDistribution, p)
@@ -215,6 +307,9 @@ def Hermite(name, a1, a2):
     r"""
     Create a discrete random variable with a Hermite distribution.
 
+    Explanation
+    ===========
+
     The density of the Hermite distribution is given by
 
     .. math::
@@ -224,8 +319,8 @@ def Hermite(name, a1, a2):
     Parameters
     ==========
 
-    a1: A Positive number greater than equal to 0.
-    a2: A Positive number greater than equal to 0.
+    a1 : A Positive number greater than equal to 0.
+    a2 : A Positive number greater than equal to 0.
 
     Returns
     =======
@@ -292,6 +387,9 @@ def Logarithmic(name, p):
     r"""
     Create a discrete random variable with a Logarithmic distribution.
 
+    Explanation
+    ===========
+
     The density of the Logarithmic distribution is given by
 
     .. math::
@@ -300,7 +398,7 @@ def Logarithmic(name, p):
     Parameters
     ==========
 
-    p: A value between 0 and 1
+    p : A value between 0 and 1
 
     Returns
     =======
@@ -319,7 +417,7 @@ def Logarithmic(name, p):
     >>> X = Logarithmic("x", p)
 
     >>> density(X)(z)
-    -5**(-z)/(z*log(4/5))
+    -1/(5**z*z*log(4/5))
 
     >>> E(X)
     -1/(-4*log(5) + 8*log(2))
@@ -331,7 +429,7 @@ def Logarithmic(name, p):
     ==========
 
     .. [1] https://en.wikipedia.org/wiki/Logarithmic_distribution
-    .. [2] http://mathworld.wolfram.com/LogarithmicDistribution.html
+    .. [2] https://mathworld.wolfram.com/LogarithmicDistribution.html
 
     """
     return rv(name, LogarithmicDistribution, p)
@@ -353,35 +451,39 @@ class NegativeBinomialDistribution(SingleDiscreteDistribution):
         r = self.r
         p = self.p
 
-        return binomial(k + r - 1, k) * (1 - p)**r * p**k
+        return binomial(k + r - 1, k) * (1 - p)**k * p**r
 
     def _characteristic_function(self, t):
         r = self.r
         p = self.p
 
-        return ((1 - p) / (1 - p * exp(I*t)))**r
+        return (p / (1 - (1 - p) * exp(I*t)))**r
 
     def _moment_generating_function(self, t):
         r = self.r
         p = self.p
 
-        return ((1 - p) / (1 - p * exp(t)))**r
-
+        return (p / (1 - (1 - p) * exp(t)))**r
 
 def NegativeBinomial(name, r, p):
     r"""
     Create a discrete random variable with a Negative Binomial distribution.
 
+    Explanation
+    ===========
+
     The density of the Negative Binomial distribution is given by
 
     .. math::
-        f(k) := \binom{k + r - 1}{k} (1 - p)^r p^k
+        f(k) := \binom{k + r - 1}{k} (1 - p)^k p^r
 
     Parameters
     ==========
 
-    r: A positive value
-    p: A value between 0 and 1
+    r : A positive value
+        Number of successes until the experiment is stopped.
+    p : A value between 0 and 1
+        Probability of success.
 
     Returns
     =======
@@ -395,25 +497,25 @@ def NegativeBinomial(name, r, p):
     >>> from sympy import Symbol, S
 
     >>> r = 5
-    >>> p = S.One / 5
+    >>> p = S.One / 3
     >>> z = Symbol("z")
 
     >>> X = NegativeBinomial("x", r, p)
 
     >>> density(X)(z)
-    1024*5**(-z)*binomial(z + 4, z)/3125
+    (2/3)**z*binomial(z + 4, z)/243
 
     >>> E(X)
-    5/4
+    10
 
     >>> variance(X)
-    25/16
+    30
 
     References
     ==========
 
     .. [1] https://en.wikipedia.org/wiki/Negative_binomial_distribution
-    .. [2] http://mathworld.wolfram.com/NegativeBinomialDistribution.html
+    .. [2] https://mathworld.wolfram.com/NegativeBinomialDistribution.html
 
     """
     return rv(name, NegativeBinomialDistribution, r, p)
@@ -440,11 +542,26 @@ class PoissonDistribution(SingleDiscreteDistribution):
     def _moment_generating_function(self, t):
         return exp(self.lamda * (exp(t) - 1))
 
+    def expectation(self, expr, var, evaluate=True, **kwargs):
+        if evaluate:
+            if expr == var:
+                return self.lamda
+            if (
+                isinstance(expr, FallingFactorial)
+                and expr.args[1].is_integer
+                and expr.args[1].is_positive
+                and expr.args[0] == var
+            ):
+                return self.lamda ** expr.args[1]
+        return super().expectation(expr, var, evaluate, **kwargs)
 
 def Poisson(name, lamda):
     r"""
     Create a discrete random variable with a Poisson distribution.
 
+    Explanation
+    ===========
+
     The density of the Poisson distribution is given by
 
     .. math::
@@ -453,7 +570,7 @@ def Poisson(name, lamda):
     Parameters
     ==========
 
-    lamda: Positive number, a rate
+    lamda : Positive number, a rate
 
     Returns
     =======
@@ -484,7 +601,7 @@ def Poisson(name, lamda):
     ==========
 
     .. [1] https://en.wikipedia.org/wiki/Poisson_distribution
-    .. [2] http://mathworld.wolfram.com/PoissonDistribution.html
+    .. [2] https://mathworld.wolfram.com/PoissonDistribution.html
 
     """
     return rv(name, PoissonDistribution, lamda)
@@ -526,6 +643,9 @@ def Skellam(name, mu1, mu2):
     r"""
     Create a discrete random variable with a Skellam distribution.
 
+    Explanation
+    ===========
+
     The Skellam is the distribution of the difference N1 - N2
     of two statistically independent random variables N1 and N2
     each Poisson-distributed with respective expected values mu1 and mu2.
@@ -538,8 +658,8 @@ def Skellam(name, mu1, mu2):
     Parameters
     ==========
 
-    mu1: A non-negative value
-    mu2: A non-negative value
+    mu1 : A non-negative value
+    mu2 : A non-negative value
 
     Returns
     =======
@@ -609,6 +729,9 @@ def YuleSimon(name, rho):
     r"""
     Create a discrete random variable with a Yule-Simon distribution.
 
+    Explanation
+    ===========
+
     The density of the Yule-Simon distribution is given by
 
     .. math::
@@ -617,7 +740,7 @@ def YuleSimon(name, rho):
     Parameters
     ==========
 
-    rho: A positive value
+    rho : A positive value
 
     Returns
     =======
@@ -679,6 +802,9 @@ def Zeta(name, s):
     r"""
     Create a discrete random variable with a Zeta distribution.
 
+    Explanation
+    ===========
+
     The density of the Zeta distribution is given by
 
     .. math::
@@ -687,7 +813,7 @@ def Zeta(name, s):
     Parameters
     ==========
 
-    s: A value greater than 1
+    s : A value greater than 1
 
     Returns
     =======
