@@ -75,7 +75,7 @@ class ArrayKind(Kind):
 
     @classmethod
     def _union(cls, kinds) -> 'ArrayKind':
-        elem_kinds = set(e.kind for e in kinds)
+        elem_kinds = {e.kind for e in kinds}
         if len(elem_kinds) == 1:
             elemkind, = elem_kinds
         else:
@@ -84,7 +84,7 @@ class ArrayKind(Kind):
 
 
 class NDimArray(Printable):
-    """
+    """N-dimensional array.
 
     Examples
     ========
@@ -171,7 +171,7 @@ class NDimArray(Printable):
 
     def _get_tuple_index(self, integer_index):
         index = []
-        for i, sh in enumerate(reversed(self.shape)):
+        for sh in reversed(self.shape):
             index.append(integer_index % sh)
             integer_index //= sh
         index.reverse()
@@ -189,7 +189,7 @@ class NDimArray(Printable):
         return None
 
     def _setter_iterable_check(self, value):
-        from sympy.matrices.matrices import MatrixBase
+        from sympy.matrices.matrixbase import MatrixBase
         if isinstance(value, (Iterable, MatrixBase, NDimArray)):
             raise NotImplementedError
 
@@ -214,7 +214,7 @@ class NDimArray(Printable):
 
     @classmethod
     def _handle_ndarray_creation_inputs(cls, iterable=None, shape=None, **kwargs):
-        from sympy.matrices.matrices import MatrixBase
+        from sympy.matrices.matrixbase import MatrixBase
         from sympy.tensor.array import SparseNDimArray
 
         if shape is None:
@@ -243,7 +243,7 @@ class NDimArray(Printable):
 
         if isinstance(iterable, (Dict, dict)) and shape is not None:
             new_dict = iterable.copy()
-            for k, v in new_dict.items():
+            for k in new_dict:
                 if isinstance(k, (tuple, Tuple)):
                     new_key = 0
                     for i, idx in enumerate(k):
@@ -362,7 +362,8 @@ class NDimArray(Printable):
 
         if self.rank() == 0:
             return printer._print(self[()])
-
+        if 0 in self.shape:
+            return f"{self.__class__.__name__}([], {self.shape})"
         return f(self._loop_size, self.shape, 0, self._loop_size)
 
     def tolist(self):
@@ -417,7 +418,7 @@ class NDimArray(Printable):
         return type(self)(result_list, self.shape)
 
     def __mul__(self, other):
-        from sympy.matrices.matrices import MatrixBase
+        from sympy.matrices.matrixbase import MatrixBase
         from sympy.tensor.array import SparseNDimArray
         from sympy.tensor.array.arrayop import Flatten
 
@@ -434,7 +435,7 @@ class NDimArray(Printable):
         return type(self)(result_list, self.shape)
 
     def __rmul__(self, other):
-        from sympy.matrices.matrices import MatrixBase
+        from sympy.matrices.matrixbase import MatrixBase
         from sympy.tensor.array import SparseNDimArray
         from sympy.tensor.array.arrayop import Flatten
 
@@ -451,7 +452,7 @@ class NDimArray(Printable):
         return type(self)(result_list, self.shape)
 
     def __truediv__(self, other):
-        from sympy.matrices.matrices import MatrixBase
+        from sympy.matrices.matrixbase import MatrixBase
         from sympy.tensor.array import SparseNDimArray
         from sympy.tensor.array.arrayop import Flatten
 
