@@ -1,3 +1,4 @@
+import string
 from itertools import zip_longest
 
 from sympy.utilities.enumerative import (
@@ -85,7 +86,7 @@ def compare_multiset_w_baseline(multiplicities):
     baseline implementation, and compare the results.
 
     """
-    letters = "abcdefghijklmnopqrstuvwxyz"
+    letters = string.ascii_lowercase
     bl_partitions = multiset_partitions_baseline(multiplicities, letters)
 
     # The partitions returned by the different algorithms may have
